#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
INVENTARI DE CNMC Centres Transformadors
"""
from __future__ import absolute_import
from datetime import datetime
import traceback

from libcnmc.utils import format_f
from libcnmc.core import StopMultiprocessBased

TIPUS_INST = {
    'TI-174': 1,
    'TI-187': 1,
    'TI-182': 1,
    'TI-183': 1,
    'TI-187A': 1,
    'TI-179': 0,
    'TI-177': 0,
    'TI-181': 2,
    'TI-102V': 0,
}

class FB2_2(StopMultiprocessBased):
    """
    Class that generates the CT file of the 4666
    """
    def __init__(self, **kwargs):
        """
        Class constructor
        :param kwargs: year(generation year), codi_r1 R1 code
        :return: CT
        """
        super(FB2_2, self).__init__(**kwargs)
        self.year = kwargs.pop('year', datetime.now().year - 1)
        self.report_name = 'FB2_2 Bis - Cel·les'
        self.base_object = 'Cel·les'
        self.fiabilitat = kwargs.get("fiabilitat")
        self.doslmesp = kwargs.get("doslmesp")

    def get_sequence(self):
        """
        Generates a list of elements of celles to be passed to the consumer

        :return: None
        """

        search_params = [
            ("installacio", "like", "%giscedata.cts%"),
            ("tipus_element.codi", "!=", "FUS_AT")
        ]


        data_pm = '{}-01-01'.format(self.year + 1)
        data_baixa = '{}-12-31'.format(self.year)
        search_params += [('criteri_regulatori', '!=', 'excloure'),
                          '|', ('data_pm', '=', False),
                          ('data_pm', '<', data_pm),
                          '|', ('data_baixa', '>', data_baixa),
                          ('data_baixa', '=', False)
                          ]
        # Revisem que si està de baixa ha de tenir la data informada.
        search_params += ['|',
                          '&', ('active', '=', False),
                          ('data_baixa', '!=', False),
                          ('active', '=', True)]
        return self.connection.GiscedataCellesCella.search(
            search_params, 0, 0, False, {'active_test': False})

    def get_codi_ct(self, ct_id):
        o = self.connection
        ct = o.GiscedataCts.read(ct_id, ['name'])
        res = ''
        if ct:
            res = ct['name']
        return res

    def get_codi_maquina(self, ct_id, o_id_cella):
        o = self.connection
        res = ''
        cella_id = o.GiscedataCellesCella.search([('name', '=', o_id_cella)], 0, 0, False, {'active_test': False})
        tipus_pos_data = o.GiscedataCellesCella.read(cella_id[0], ['tipus_posicio'])
        if tipus_pos_data.get('tipus_posicio', False):
            tipus_pos_id = tipus_pos_data['tipus_posicio'][0]
            codi_pos = o.GiscedataCellesTipusPosicio.read(tipus_pos_id, ['codi'])
            if codi_pos['codi'] == 'P':
                trafos = o.GiscedataCts.read(
                    ct_id, ['transformadors'])['transformadors']
                trafo_id = o.GiscedataTransformadorTrafo.search(
                    [('id', 'in', trafos), ('id_estat.codi', '=', 1)], 0, 1)
                res = ''
                if trafo_id:
                    codi_maquina = o.GiscedataTransformadorTrafo.read(trafo_id[0],
                                                                      ['name'])
                    res = codi_maquina['name']
        return res

    def get_tipus_inst(self, ti_cnmc_id):
        o = self.connection
        ti_cnmc = o.GiscedataTipusInstallacio.read(
            ti_cnmc_id[0], ['name'])['name']
        return ti_cnmc

    def consumer(self):
        o = self.connection
        fields_to_read = [
            'installacio', 'name', 'propietari', 'data_pm', 'cini', 'tipus_instalacio_cnmc_id'
        ]
        while True:
            try:
                # generar linies
                item = self.input_q.get()
                if item == 'STOP':
                    self.input_q.task_done()
                    break
                self.progress_q.put(item)
                celles = o.GiscedataCellesCella.read(
                    item, fields_to_read
                )

                o_ct_id = int(celles['installacio'].split(',')[1])
                o_ct = self.get_codi_ct(o_ct_id)

                o_id_cella = celles['name']
                o_maquina = self.get_codi_maquina(o_ct_id, o_id_cella)
                o_cini = celles['cini'] or ''
                o_propietari = int(celles['propietari'])
                if o_propietari == 0:
                    o_maquina = ''
                o_interruptor = self.get_tipus_inst(celles['tipus_instalacio_cnmc_id'])
                o_interruptor_val = TIPUS_INST[o_interruptor]

                o_data = ''
                if celles['data_pm']:
                    o_data = datetime.strptime(celles['data_pm'], "%Y-%m-%d")
                    o_data = int(o_data.year)


                self.output_q.put([
                    o_ct,                # CT
                    o_id_cella,          # IDENTIFICADOR_CELDA
                    o_maquina,           # IDENTIFICADOR_MAQUINA
                    o_cini,              # CINI
                    o_interruptor_val,   # INTERRUPTOR
                    o_propietari,        # PROPIEDAD
                    o_data,              # FECHA PUESTA EN SERVICIO
                ])
                self.input_q.task_done()
            except Exception:
                self.input_q.task_done()
                traceback.print_exc()
                if self.raven:
                    self.raven.captureException()



# -*- coding: utf-8 -*-
from libcnmc.core import UpdateFile

class UpdateCINISTrafo(UpdateFile):
    def __init__(self, **kwargs):
        super(UpdateCINISTrafo, self).__init__(**kwargs)
        self.header = [
            'num_transformador', 'cini'
        ]
        self.search_keys = [('num_transformador', 'name')]
        self.object = self.connection.GiscedataTransformadorTrafo



# -*- coding: utf-8 -*-
from libcnmc.core import StopMultiprocessBased
from libcnmc.utils import format_f, convert_srid, get_srid, get_ine
from datetime import datetime
from shapely import wkt
import traceback


class FC5(StopMultiprocessBased):

    def __init__(self, **kwargs):
        super(FC5, self).__init__(**kwargs)
        self.year = kwargs.pop('year', datetime.now().year - 1)
        self.codi_r1 = kwargs.pop('codi_r1')
        self.report_name = 'Formulario C5: Información relativa a operaciones con partes vinculadas'
        self.base_object = 'Operaciones partes vinculadas'

    def get_sequence(self):
        c5_ids = self.connection.model('cir8.2021.c5').search([('year', '=', self.year)])
        return c5_ids

    def consumer(self):
        O = self.connection
        fields_to_read = ['cod_operacion', 'parte_vinculada', 'cuenta', 'cuenta_pgc', 'ceco', 'coco', 'razon_social',
                          'cif_nif', 'descripcion', 'margen', 'gasto', 'ingreso']
        while True:
            try:
                item = self.input_q.get()
                if item == 'STOP':
                    self.input_q.task_done()
                    break
                self.progress_q.put(item)

                c5 = O.model('cir8.2021.c5').read(item, fields_to_read)

                # COD_OPERACION
                o_cod_operacion = ''
                if c5.get('cod_operacion', False):
                    o_cod_operacion = c5['cod_operacion']

                # PARTE_VINCULADA
                o_parte_vinculada = ''
                if c5.get('parte_vinculada', False):
                    o_parte_vinculada = c5['parte_vinculada']

                # CUENTA
                o_cuenta = ''
                if c5.get('cuenta', False):
                    o_cuenta = c5['cuenta']

                # CUENTA_PGC
                o_cuenta_pgc = ''
                if c5.get('cuenta_pgc', False):
                    o_cuenta_pgc = c5['cuenta_pgc']

                # CECO
                o_ceco = ''
                if c5.get('ceco', False):
                    o_ceco = c5['ceco']

                # COCO
                o_coco = ''
                if c5.get('coco', False):
                    o_coco = c5['coco']

                # RAZON_SOCIAL
                o_razon_social = ''
                if c5.get('razon_social', False):
                    o_razon_social = c5['razon_social']

                # CIF_NIF
                o_cif_nif = ''
                if c5.get('cif_nif', False):
                    o_cif_nif = c5['cif_nif']

                # DESCRIPCION
                o_descripcion = ''
                if c5.get('descripcion', False):
                    o_descripcion = c5['descripcion']

                # MARGEN
                o_margen = ''
                if c5.get('margen', False):
                    o_margen = c5['margen']

                # GASTO
                o_gasto = ''
                if c5.get('gasto', False):
                    o_gasto = c5['gasto']

                # INGRESO
                o_ingreso = ''
                if c5.get('ingreso', False):
                    o_ingreso = c5['ingreso']

                self.output_q.put([
                    o_cod_operacion,                    # COD_OPERACION
                    o_parte_vinculada,                  # PARTE_VINCULADA
                    o_cuenta,                           # CUENTA
                    o_cuenta_pgc,                       # CUENTA_PGC
                    o_ceco,                             # CECO
                    o_coco,                             # COCO
                    o_razon_social,                     # RAZON_SOCIAL
                    o_cif_nif,                          # CIF_NIF
                    o_descripcion,                      # DESCRIPCION
                    format_f(o_margen, decimals=2),     # MARGEN
                    format_f(o_gasto, decimals=2),      # GASTO
                    format_f(o_ingreso, decimals=2),    # INGRESO
                ])
                self.input_q.task_done()
            except Exception:
                self.input_q.task_done()
                traceback.print_exc()
                if self.raven:
                    self.raven.captureException()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
INVENTARI DE CNMC AT
"""
import traceback

from libcnmc.res_4667.utils import get_resum_any_id
from libcnmc.utils import get_name_ti, get_codigo_ccaa, format_f
from libcnmc.core import MultiprocessBased


class LAT(MultiprocessBased):
    """
    Class to generate F1 of 4667
    """

    def __init__(self, **kwargs):
        """
        Class constructor
        :param kwargs: 
        """
        self.year = kwargs.pop('year')
        super(LAT, self).__init__(**kwargs)

    def get_sequence(self):
        """
        Generates the sequence of ids to make the report
        
        :return: List of ids
        :rtype: list
        """
        id_resum = get_resum_any_id(self.connection, self.year)

        search_linia = [("resums_inversio", "in", id_resum)]
        return self.connection.GiscedataCnmcLinies.search(search_linia)

    def consumer(self):
        """
        Generates the line of the file
        :return: Line 
        :rtype: str
        """

        O = self.connection
        fields_to_read = [
            "codi", "finalitat", "id_instalacio", "codi_tipus_inst", "cini",
            "ccaa", "ccaa_2", "any_apm", "capacidad_prv", "long_total",
            "vol_total_inv", "ajudes", "inv_financiada", "vpi_retri", "estado",
            "actuacio_elegible_prtr"
        ]

        while True:
            try:
                item = self.input_q.get()
                self.progress_q.put(item)

                linia = O.GiscedataCnmcLinies.read(item, fields_to_read)
                if linia["ccaa"]:
                    ccaa = linia["ccaa"][0]
                else:
                    ccaa = False

                if linia["ccaa_2"]:
                    ccaa_2 = linia["ccaa_2"][0]
                else:
                    ccaa_2 = False

                output = [
                    linia["codi"][1],
                    linia["finalitat"],
                    linia["id_instalacio"],
                    linia["cini"],
                    get_name_ti(O, linia["codi_tipus_inst"][0]),
                    get_codigo_ccaa(O, ccaa),
                    get_codigo_ccaa(O, ccaa_2),
                    linia["any_apm"],
                    format_f(linia["long_total"], 3) or "0.00",
                    linia["capacidad_prv"],
                    format_f(linia["vol_total_inv"], 2) or "0.00",
                    format_f(linia["ajudes"], 2) or "0.00",
                    format_f(linia["inv_financiada"], 2) or "0.000",
                    format_f(linia["vpi_retri"], 2) or "0.00",
                    linia["estado"],
                    linia["actuacio_elegible_prtr"] or '',
                ]
                self.output_q.put(output)

            except Exception:
                traceback.print_exc()
                if self.raven:
                    self.raven.captureException()
            finally:
                self.input_q.task_done()



from __future__ import absolute_import
from libcnmc.models.cnmcmodel import CNMCModel

from collections import OrderedDict

from libcnmc.models.fields import String, Integer, Decimal


class F6Res4666(CNMCModel):
    """
        Class for sixth file of resolution 4666(Despatxos)
    """

    schema = OrderedDict([
        ('identificador', String()),
        ('cini', String()),
        ('denominacion', String()),
        ('fecha_aps', String()),
        ('fecha_baja', String()),
        ('valor_inversion', Decimal(3)),
        ('estado', Integer())
    ])

    @property
    def ref(self):
        return self.store.identificador

    def __cmp__(self, other):
        comp_fields = ['cini', 'fecha_aps']
        if self.diff(other, comp_fields):
            return True
        else:
            return False








from __future__ import absolute_import
from libcnmc.models.cnmcmodel import CNMCModel
from collections import OrderedDict

from libcnmc.models.fields import String, Integer, Decimal


class F5Res4771(CNMCModel):
    """
        Class for fifth file of resolution 4771(Maquines)
    """
    schema = OrderedDict([
        ('identificador', String()),
        ('cini', String()),
        ('denominacion', String()),
        ('codigo_tipo_maquina', String()),
        ('codigo_ccaa', Integer()),
        ('tension_primario', Decimal(2)),
        ('tension_secundario', Decimal(2)),
        ('participacion', Decimal(2)),
        ('fecha_aps', String()),
        ('capacidad', Decimal(2))
    ])

    @property
    def ref(self):
        return self.store.identificador

    def __cmp__(self, other):
        comp_fields = [
            'cini', 'tension_primario', 'tension_secundario',
            'capacidad', 'denominacion',
        ]
        if self.diff(other, comp_fields):
            return True
        else:
            return False








# -*- coding: utf-8 -*-
import os
import multiprocessing
import tempfile

from shapely import wkt
from shapely.geometry import Point
from shapely.ops import transform
import pyproj

AVAILABLE_CPUS = max(multiprocessing.cpu_count() - os.getloadavg()[0], 0) + 1

N_PROC = max(int(os.getenv('N_PROC', 0)), AVAILABLE_CPUS)

from libcnmc.core.backend import OOOPFactory

CODIS_TARIFA = {
    '2.0A': 416,
    '2.0DHA': 417,
    '2.1A': 418,
    '2.1DHA': 419,
    '2.0DHS': 426,
    '2.1DHS': 427,
    '3.0A': 403,
    '3.1A': 404,
    '3.1A LB': 404,
    '6.1': 405,
    '6.1A': 441,
    '6.1B': 442,
    '6.2': 406,
    '6.2A': 406,
    '6.2B': 406,
    ###
    '2.0TD': 451,
    '3.0TD': 452,
    '6.1TD': 453,
    '6.2TD': 454,
    '6.3TD': 455,
    '6.4TD': 456,
    '3.0TDVE': 471,
    '6.1TDVE': 472
}


CODIS_ZONA = {'RURALDISPERSA': 'RD', 'RURALCONCENTRADA': 'RC',
              'SEMIURBANA': 'SU', 'URBANA': 'U'}


CINI_TG_REGEXP = 'I310.[23]0.$'

TENS_NORM = []

INES = {}

TARIFAS_AT = [
    '3.1A', '3.1A LB', '6.1', '6.1A', '6.1B', '6.2', '6.2A', '6.2B'
]

TARIFAS_BT = [
    '2.0A', '2.0DHA', '2.1A', '2.1DHA', '2.0DHS', '2.1DHS', '3.0A'
]

SUPPORTED_FORMS_CIR_8_2021 = [
    ('a1', 'A1'),
    ('a2', 'A2'),
    ('a3', 'A3'),
    ('a4', 'A4'),
    ('a5', 'A5'),
    ('b1', 'B1'),
    ('b1_1', 'B1.1'),
    ('b2', 'B2'),
    ('b2_1', 'B2.1'),
    ('b2_2', 'B2.2'),
    ('b3', 'B3'),
    ('b3_1', 'B3.1'),
    ('b3_2', 'B3.2'),
    ('b3_3', 'B3.3'),
    ('b4', 'B4'),
    ('b5', 'B5'),
    ('b6', 'B6'),
    ('b7', 'B7'),
    ('b8', 'B8'),
    ('b9', 'B9'),
    ('c1', 'C1'),
    ('c2', 'C2'),
    ('c3', 'C3'),
    ('c4', 'C4'),
    ('c5', 'C5'),
    ('c6', 'C6'),
    ('c7', 'C7'),
    ('d1', 'D1'),
    ('d2', 'D2'),
    ('f1', 'F1'),
]

PROJS = {
        '25829': pyproj.CRS('EPSG:25829'),
        '25830': pyproj.CRS('EPSG:25830'),
        '25831': pyproj.CRS('EPSG:25831'),
}

PROJ_TRANSFORMERS = {
        '25829': pyproj.Transformer.from_crs(
            PROJS['25829'], PROJS['25830']
        ).transform,
        '25830': pyproj.Transformer.from_crs(
            PROJS['25830'], PROJS['25830']
        ).transform,
        '25831': pyproj.Transformer.from_crs(
            PROJS['25831'], PROJS['25830']
        ).transform,
}

def get_forced_elements(connection, model):
    """
    Returns the force include and force exclude ids of elements
    :param connection:
    :param model:

    :return:Included and excluded elements
    :rtype: dict(str,list)
    """
    c = connection
    try:
        mod_obj = getattr(c, c.normalize_model_name(model))
    except Exception:
        mod_obj = c.model(model)

    include_search_params = [("criteri_regulatori", "=", "incloure")]
    include_ids = mod_obj.search(
        include_search_params, False, False, False, {'active_test': False}
    )

    include_search_params = [("criteri_regulatori", "=", "excloure")]
    exclude_ids = mod_obj.search(
        include_search_params, False, False, False, {'active_test': False}
    )

    return {
        "include": include_ids,
        "exclude": exclude_ids
    }


def fetch_cts_node(connection):
    """
    Gets the nodes of the CTs

    :param connection: OpenERP connection
    :return: Ct id, node name
    :rtype: dict
    """
    try:
        obj_ct = connection.GiscedataCts
        ids_cts = obj_ct.search([])
        data_cts = obj_ct.read(ids_cts,["node_id"])
        return {data["id"]:data["node_id"] for data in data_cts}
    except Exception:
        ids_blk = connection.GiscegisBlocsCtat.search([])
        read_fields = ["ct", "node"]
        data_cts = connection.GiscegisBlocsCtat.read(ids_blk, read_fields)
        result_dict = dict.fromkeys(ids_blk)
        for ct in data_cts:
            result_dict[ct["ct"][0]] = ct.get("node",['',''])[1]
        return result_dict


def fetch_prov_ine(connection):
    """
    Fetch the INE code of the provincias

    :param connection: OpenERP connection
    :return: provnice id, INE code
    :rtype: dict
    """

    ids_prov = connection.ResCountryState.search([])
    data_prov = connection.ResCountryState.read(ids_prov, ['code'])
    dict_prov = dict.fromkeys(ids_prov)
    for prov in data_prov:
        dict_prov[prov["id"]] = prov["code"]
    return dict_prov


def fetch_mun_ine(connection):
    """
    Fetch teh INE code of the municipio

    :param connection:
    :return:
    :rtype: dict
    """

    ids_mun = connection.ResMunicipi.search([])

    data_mun = connection.ResMunicipi.read(ids_mun, ['ine', 'dc'])
    ret_vals = dict.fromkeys(ids_mun)
    for mun in data_mun:
        ret_vals[mun["id"]] = '{0}{1}'.format(mun['ine'][-3:], mun['dc'])
    return ret_vals


def fetch_tensions_norm(connection):
    """
    Gets all the tensions

    :param connection:
    :return: Id and tensio
    :rtype: dict
    """
    t_ids = connection.GiscedataTensionsTensio.search([])
    t_data = connection.GiscedataTensionsTensio.read(t_ids, ["tensio"])
    d_out = dict.fromkeys(t_ids)
    for line in t_data:
        d_out[line["id"]] = format_f(float(line["tensio"]) / 1000.0, decimals=3)
    return d_out


def get_norm_tension(connection, tension):
    """
    Returns the tension normalizada from a given tensio

    :param connection: OpenERP connection
    :param tension: Tensio
    :return: Tensio normalitzada
    """

    if not TENS_NORM:
        tension_fields_to_read = ['l_inferior', 'l_superior', 'tensio']
        tension_vals = connection.GiscedataTensionsTensio.read(
            connection.GiscedataTensionsTensio.search([]),
            tension_fields_to_read)
        TENS_NORM.extend([(t['l_inferior'], t['l_superior'], t['tensio'])
                          for t in tension_vals])
    if not tension:
        return tension

    for t in TENS_NORM:
        if t[0] <= tension < t[1]:
            return t[2]

    return tension


def get_name_ti(connection, ti):
    """
    Returns the name of the TI

    :param connection: Database connection
    :param ti: Id of the TI
    :type ti: int
    :return: Name of the TI
    :rtype: str
    """
    if ti:
        data = connection.GiscedataTipusInstallacio.read([ti], ["name"])[0]
        if data:
            return data["name"]
        else:
            return ""
    else:
        return ""


def get_codi_actuacio(connection, actuacio):
    """
    Returns the code of the actuacio

    :param connection: Database connection
    :param actuacio: Id of the Actuacio
    :type actuacio: int
    :return: Code of the Actuacio
    :rtype: str
    """
    code = ''
    if actuacio:
        data = connection.GiscedataActuacio.read([actuacio], ["codi"])[0]
        if data:
            code = data["codi"]

    return code


def get_codigo_ccaa(connection, ccaa):
    """
    Return the code from CCAA

    :param connection: Database conection
    :param ccaa: Id of the CCAA
    :return: Codigo CCAA
    :rtype: int, str
    """
    if ccaa:
        data = connection.ResComunitat_autonoma.read(ccaa, ["codi"])
        if data:
            return data["codi"]
        else:
            return ''
    else:
        return ''


def get_ine(connection, ine):
    """
    Retornem dos valors el codi de l'estat i el codi ine sense estat.

    :param connection: OpenERP connection
    :param ine:
    :return: State, ine municipi
    :rtype: tuple
    """
    if not INES:
        ids = connection.ResMunicipi.search([('dc', '!=', False)])
        for municipi in connection.ResMunicipi.read(ids, ['ine', 'dc']):
            INES[municipi['ine']] = municipi['dc']
    # Accedim directament per la clau així si peta rebrem un sentry.
    if ine not in INES:
        state = ''
        ret_ine = ''
    else:
        state = ine[:2]
        ret_ine = ine[2:] + INES[ine]
    return state, ret_ine


def get_comptador(connection, polissa_id, year):
        comp_obj = connection.GiscedataLecturesComptador
        comp_id = comp_obj.search([
            ('polissa', '=', polissa_id),
            ('data_alta', '<', '{}-01-01'.format(year + 1))
        ], 0, 1, 'data_alta desc', {'active_test': False})
        return comp_id


def get_id_expedient(connection, expedients_id):
    id_expedient = False
    if expedients_id:
        search_params = [
            ('id', 'in', expedients_id), ('industria_data', '!=', False)]
        id_expedients = connection.GiscedataExpedientsExpedient.search(
            search_params, 0, 1, 'industria_data desc')
        if id_expedients:
            id_expedient = id_expedients[0]
    return id_expedient


def get_id_municipi_from_company(connection, get_name=False):
    O = connection
    id_municipi = False
    # Si no hi ha ct agafem la comunitat del rescompany
    company_partner = O.ResCompany.read(1, ['partner_id'])
    # funció per trobar la ccaa desde el municipi
    if company_partner:
        partner_address = O.ResPartner.read(
            company_partner['partner_id'][0], ['address'])
        address = O.ResPartnerAddress.read(
            partner_address['address'][0], ['id_municipi'])
        if address['id_municipi']:
            id_municipi = address['id_municipi'][0]
        if get_name:
            name_municipi = id_municipi = address['id_municipi'][1]
            return  id_municipi, name_municipi
    return id_municipi


def tallar_text(text, longitud):
    try:
        if len(text) > longitud:
            return text[:longitud-3] + '...'
        else:
            return text
    except TypeError:
        return ''


def format_f(num, decimals=2):
    """
    Formats float with comma decimal separator

    :param num:
    :param decimals:
    :return:
    """

    if isinstance(num, float):
        fstring = '%%.%df' % decimals
        return (fstring % num).replace('.', ',')
    return num


def format_f_6181(num, decimals=2, float_type=None):
    """
    Formats float with comma decimal separator according to 6181 resolution specs

    :param num:
    :param decimals:
    :param float_type: type of float the BOE defines.
                       It can be either 'euro', which has 2 decimals, or
                       'decimal', which has 3.
    :return:
    """

    if isinstance(float_type, str):
        decimals = 2 if float_type == 'euro' else 3 if float_type == 'decimal' else decimals

    return format_f(num, decimals=decimals)


def format_ccaa_code(ccaa):
    """
    E.g. Catalonia has ccaa code: 09.
    If *ccaa* input is '9', the output will be '09'.
    """
    if not ccaa:
        return ''
    if isinstance(ccaa, int):
        ccaa = str(ccaa)
    return ccaa.zfill(2)


def convert_spanish_date(date):
    from datetime import datetime
    try:
        date_parsed = datetime.strptime(str(date), '%Y-%m-%d')
        date = date_parsed.strftime('%d/%m/%Y')
    except:
        pass
    return date


def convert_srid(srid_source, point):
    """
    Converts the projection of the point from srid_source to 25830
    :param srid_source: Source SRID
    :param point: Point to convert
    :type point: list or tuple
    :return: Converted point
    :rtype: tuple
    """
    assert srid_source in ['25829', '25830', '25831']
    dpoint = Point(point[0], point[1])
    transformer = PROJ_TRANSFORMERS[srid_source]

    dpoint = transform(transformer, dpoint)

    return (dpoint.x, dpoint.y)


def get_srid(c):
    """
    Returns the SRID from the configuration

    :param c: OpenERP connection
    :return: SRID
    :rtype: str
    """

    return str(c.GiscegisBaseGeom.get_srid())


def merge_procs(procs, **kwargs):
    """
    Generates multiple procs and merges the results

    :param procs: Procs to generate
    :type procs: list[MultiprocessBased]
    :param database: OpenERP Database
    :type database: str
    :param user: OpenERP user
    :type user: str
    :param password: OpenERP password
    :type pasword: str
    :param port: OpenERP port
    :param server: OpenERP server to connet
    :type server: str
    :return: Result of the procs
    :rtype: str
    """

    O = OOOPFactory(dbname=kwargs['database'], user=kwargs['user'],
                    pwd=kwargs['password'], port=kwargs['port'],
                    uri=kwargs['server'])

    data_out = ""
    original_out_url = kwargs["output"]
    for proc_fnc in procs:
        temp_fd = tempfile.NamedTemporaryFile()
        tmp_url = temp_fd.name
        temp_fd.close()

        proc_kwargs = kwargs
        proc_kwargs["connection"] = O
        proc_kwargs["output"] = tmp_url
        proc = proc_fnc(**proc_kwargs)
        proc.calc()

        with open(tmp_url, 'r') as fd:
            data_out += fd.read()

    with open(original_out_url, "w") as fd_out:
        fd_out.write(data_out)


def adapt_diff(diff):
    """
    Formats diff to human-readable format
    :param diff: dictionary
    :type diff: dict
    :return: result of the formatted text
    :rtype: str
    """

    a = ""
    for key, value in diff.items():
        a += "[{}] nou:{} antic:{} ".format(key, *value)
    return a


def parse_geom(geom):
    """
    Parse the input Geom to a list of dicts with the coordinates that compose it
    :param geom: Geometry on WKT: 'LINESTRING(X1 Y2,...,Xn Yn)', 'POINT(X Y)'
    :type geom: str
    :return: The Points that compose theinput Geometry
    :rtype list of dict[str,str] or []
    """

    if geom:
        points = [
            {
                'x': coord[0],
                'y': coord[1]
            } for coord in wkt.loads(geom).coords
        ]
    else:
        points = []

    return points


def get_ines(o, ids):
    """
    Gets de INE code of 'provincia' and 'municipi' by its ids
    :param o: OpenERP connection
    :param ids: A single 'provincia' id and 'municipi' id
    :type ids: dict
    :return: The INE code of each id
    :rtype: dict
    """
    res = {'ine_municipi': 0, 'ine_provincia': 0}
    if ids.get('id_municipi', False):
        municipi_dict = o.ResMunicipi.read(ids['id_municipi'][0],
                                           ['ine', 'dc'])
        res['ine_municipi'] = '{0}{1}'.format(municipi_dict['ine'][-3:],
                                              municipi_dict['dc'])
    if ids.get('id_provincia', False):
        res['ine_provincia'] = o.ResCountryState.read(
            ids['id_provincia'][0], ['code']
        )['code']
    return res


def get_tipus_connexio(o, id_escomesa):
    """
    Gets the tipus of connexio of an escomesa. If it's a BT escomesa we
    check the TramBT that suplies it to see if its aerial or underground.
    If it's not a BT escomesa we directly set the tipus of connexio to
    aerial.
    :param id_escomesa: Id of the escomesa
    :type id_escomesa: int
    :return: A or S depending on if the linia that suplies the escomesa
             is aerial or underground
    :rtype: str
    """

    tipus = 'A'
    if 'node_id' in o.GiscedataCupsEscomesa.fields_get().keys() and 'edge_id' in o.GiscedataBtElement.fields_get().keys():
        node_id = o.GiscedataCupsEscomesa.read(
            id_escomesa, ['node_id']
        )['node_id']
        if node_id:
            edge_id = o.GiscegisEdge.search(
                [
                    '|',
                    ('start_node', '=', node_id[0]),
                    ('end_node', '=', node_id[0])
                ]
            )
            if edge_id:
                tram_bt = o.GiscedataBtElement.search(
                    [('edge_id', '=', edge_id[0])]
                )
                if tram_bt:
                    tram_bt = o.GiscedataBtElement.read(
                        tram_bt[0], ['tipus_linia']
                    )
                    if tram_bt['tipus_linia']:
                        tipus = tram_bt['tipus_linia'][1][0]
    else:
        bloc = o.GiscegisBlocsEscomeses.search(
            [('escomesa', '=', id_escomesa)]
        )
        if bloc:
            bloc = o.GiscegisBlocsEscomeses.read(bloc[0], ['node'])
            if bloc['node']:
                node = bloc['node'][0]
                edge_bt = o.GiscegisEdge.search(
                    [
                        '|',
                        ('start_node', '=', node),
                        ('end_node', '=', node),
                        '|',
                        ('layer', 'ilike', self.layer),
                        ('layer', 'ilike', 'EMBARRA%BT%')
                    ]
                )
                edge = o.GiscegisEdge.read(
                    edge_bt[0], ['id_linktemplate']
                )
                if edge['id_linktemplate']:
                    tram_bt = o.GiscedataBtElement.search(
                        [
                            ('name', '=', edge['id_linktemplate'])
                        ]
                    )
                    if tram_bt:
                        tram_bt = o.GiscedataBtElement.read(
                            tram_bt[0], ['tipus_linia']
                        )
                        if tram_bt:
                            if tram_bt['tipus_linia']:
                                tipus = tram_bt['tipus_linia'][1][0]

    return tipus


def get_serveis_aux(o, re_id):
    '''
    Retorna el SSAA de una instal·lació RECORE. Primer mira si en té alguna associada, sinó la busca
    :param o: Connexió
    :param re_id: ID Instal·lació RECORE
    :return: Tupla (id, name) del CUPS SSAA
    '''
    serveis_aux = ''
    re_obj = o.GiscedataRe
    re_cups = re_obj.read(re_id, ['cups'])['cups']
    re_ssaa = re_obj.read(re_id, ['cups_serveis_aux_id'])

    if re_ssaa.get('cups_serveis_aux_id', False):
        serveis_aux = re_ssaa['cups_serveis_aux_id']
    else:
        polissa_obj = o.GiscedataPolissa
        polissa_id = polissa_obj.search([('re_installation_id', '=', re_id)])
        if polissa_id:
            polissa_data = polissa_obj.read(polissa_id[0], ['cups'])
            if polissa_data.get('cups', False):
                serveis_aux = polissa_data['cups']
        else:
            cups_20 = re_cups[1][0:20]
            polissa_id_20 = polissa_obj.search([('cups', 'like', cups_20), ('contract_type', '=', '05')])
            if polissa_id_20:
                cups_data = polissa_obj.read(polissa_id_20[0], ['cups'])
                if cups_data.get('cups', False):
                    serveis_aux = cups_data['cups']

    return serveis_aux



from __future__ import absolute_import
from collections import OrderedDict

from libcnmc.models.cnmcmodel import CNMCModel
from libcnmc.models.fields import String, Integer, Decimal


class F2Res4771(CNMCModel):
    """
        Class for second file of resolution 4771(BT)
    """

    schema = OrderedDict([
        ('identificador', String()),
        ('cini', String()),
        ('origen', String()),
        ('destino', String()),
        ('codigo_tipo_linea', String()),
        ('codigo_ccaa_1', Integer()),
        ('codigo_ccaa_2', Integer()),
        ('participacion', String()),
        ('fecha_aps', String()),
        ('numero_circuitos', Integer()),
        ('numero_conductores', Integer()),
        ('nivel_tension', Decimal(2)),
        ('longitud', Decimal(3)),
        ('intensidad_maxima', Decimal(2)),
        ('seccion', Decimal(2)),
        ('capacidad', Decimal(2)),
        ('propiedad', String())
    ])

    @property
    def ref(self):
        return self.store.identificador[1:]

    def __cmp__(self, other):
        comp_fields = ['longitud', 'cini', 'seccion', 'capacidad']
        if self.diff(other, comp_fields):
            return True
        else:
            return False








#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
INVENTARI DE CNMC BT
"""
from __future__ import absolute_import
from datetime import datetime
import traceback
import math
import sys

from libcnmc.core import StopMultiprocessBased
from libcnmc.utils import format_f, tallar_text
from libcnmc.models import F2Res4666

QUIET = False


class LBT(StopMultiprocessBased):
    """
    Class that generates the LBT(2) file of the 4666
    """
    def __init__(self, **kwargs):
        """
        Class constructor
        :param kwargs: year(generation year), codi_r1 R1 code
        :return: CT
        """
        super(LBT, self).__init__(**kwargs)
        self.year = kwargs.pop("year", datetime.now().year - 1)
        self.codi_r1 = kwargs.pop("codi_r1")
        self.base_object = "Línies BT"
        self.report_name = "CNMC INVENTARI BT"
        self.embarrats = kwargs.pop("embarrats", False)
        self.compare_field = kwargs["compare_field"]

    def get_sequence(self):
        """
        Method that generates a list of ids to pass to the consummer
        :return: List of ids
        """
        data_pm = '{0}-01-01'.format(self.year + 1)
        data_baixa = '{0}-01-01'.format(self.year)
        search_params = []
        if not self.embarrats:
            search_params += [('cable.tipus.codi', '!=', 'E')]
        search_params += [('propietari', '=', True),
                          '|', ('data_pm', '=', False),
                               ('data_pm', '<', data_pm),
                          '|', ('data_baixa', '>', data_baixa),
                               ('data_baixa', '=', False),
                          ]
        # Revisem que si està de baixa ha de tenir la data informada.
        search_params += ['|',
                          '&', ('active', '=', False),
                               ('data_baixa', '!=', False),
                          ('active', '=', True)]
        return self.connection.GiscedataBtElement.search(
            search_params, 0, 0, False, {'active_test': False})

    def consumer(self):
        """
        Method that generates the csb file
        :return: List of arrays
        """
        O = self.connection
        count = 0
        fields_to_read = [
            'name', 'municipi', 'data_pm', 'ct', 'coeficient', 'cini',
            'perc_financament', 'longitud_cad', 'cable', 'voltatge',
            'data_alta', 'propietari', 'tipus_instalacio_cnmc_id', 'baixa',
            'data_baixa', self.compare_field
        ]
        data_baixa_limit = '{0}-01-01'.format(self.year)
        data_pm_limit = '{0}-01-01'.format(self.year + 1)
        error_msg = "**** ERROR: l'element {0} (id:{1}) no està en giscegis_edges.\n"
        error_msg_multi = "**** ERROR: l'element {0} (id:{1}) està més d'una vegada a giscegis_edges. {2}\n"
        while True:
            try:
                count += 1
                item = self.input_q.get()
                if item == "STOP":
                    self.input_q.task_done()
                    break
                self.progress_q.put(item)

                linia = O.GiscedataBtElement.read(item, fields_to_read)

                res = O.GiscegisEdge.search([('id_linktemplate', '=',
                                              linia['name']),
                                             ('layer', 'ilike', '%BT%')])
                ct_name = linia['ct'][1]
                zona = O.GiscedataCts.read(
                    linia['ct'][0], ['zona_id']
                )['zona_id'][1]
                municipi = linia['municipi'][1]
                if not res:
                    if not QUIET:
                        sys.stderr.write(
                            error_msg.format(linia['name'], linia['id']))
                        sys.stderr.flush()
                    edge = {'start_node': (0, '{0}_0'.format(linia['name'])),
                            'end_node': (0, '{0}_1'.format(linia['name']))}
                elif len(res) > 1:
                    if not QUIET:
                        sys.stderr.write(
                            error_msg_multi.format(linia['name'], linia['id'], res))
                        sys.stderr.flush()
                    edge = {'start_node': (0, '{0}_0'.format(linia['name'])),
                            'end_node': (0, '{0}_1'.format(linia['name']))}
                else:
                    edge = O.GiscegisEdge.read(res[0], ['start_node',
                                                        'end_node'])
                comunitat = ''
                if linia['municipi']:
                    ccaa_obj = O.ResComunitat_autonoma
                    id_comunitat = ccaa_obj.get_ccaa_from_municipi(
                        linia['municipi'][0])
                    id_comunitat = id_comunitat[0]
                    comunidad = ccaa_obj.read(id_comunitat, ['codi'])
                    if comunidad:
                        comunitat = comunidad['codi']
                data_pm = ''
                if linia['data_pm']:
                    data_pm_linia = datetime.strptime(str(linia['data_pm']),
                                                      '%Y-%m-%d')
                    data_pm = data_pm_linia.strftime('%d/%m/%Y')
                data_baixa = ''
                if linia['baixa'] and linia['data_baixa']:
                    data_baixa = datetime.strptime(str(linia['data_baixa']),
                                                      '%Y-%m-%d')
                    data_baixa = data_baixa.strftime('%d/%m/%Y')

                # Coeficient per ajustar longituds de trams
                coeficient = linia['coeficient'] or 1.0

                try:
                    tensio = (int(linia['voltatge']) / 1000.0)
                except Exception:
                    tensio = 0.0

                propietari = linia['propietari'] and '1' or '0'

                if linia['tipus_instalacio_cnmc_id']:
                    id_ti = linia.get('tipus_instalacio_cnmc_id')[0]
                    codi_ccuu = O.GiscedataTipusInstallacio.read(
                        id_ti,
                        ['name'])['name']
                else:
                    codi_ccuu = ''

                # Agafem el cable de la linia
                if linia['cable']:
                    cable = O.GiscedataBtCables.read(linia['cable'][0], [
                        'intensitat_admisible', 'seccio'])
                else:
                    cable = {'seccio': 0, 'intensitat_admisible': 0}

                intensitat = cable['intensitat_admisible']
                # Capacitat
                capacitat = round(
                    (cable['intensitat_admisible'] * int(linia['voltatge'])
                     * math.sqrt(3)) / 1000, 3)

                if not capacitat:
                    capacitat = 1.0

                # Descripció
                origen = tallar_text(edge['start_node'][1], 50)
                final = tallar_text(edge['end_node'][1], 50)

                longitud = round(linia['longitud_cad'] * coeficient / 1000.0,
                                 3) or 0.001
                if linia['data_baixa']:
                    if linia['data_baixa'] > data_pm_limit:
                        fecha_baja = ''
                    else:
                        tmp_date = datetime.strptime(
                            linia['data_baixa'], '%Y-%m-%d')
                        fecha_baja = tmp_date.strftime('%d/%m/%Y')
                else:
                    fecha_baja = ''

                if linia[self.compare_field]:
                    last_data = linia[self.compare_field]
                    entregada = F2Res4666(**last_data)
                    actual = F2Res4666(
                        'B{0}'.format(linia['name']),
                        linia['cini'],
                        origen or '',
                        final or '',
                        codi_ccuu or '',
                        comunitat,
                        comunitat,
                        format_f(100.0 - float(linia['perc_financament']),3),
                        data_pm,
                        data_baixa,
                        1,
                        1,
                        format_f(tensio, 3),
                        format_f(longitud, 3),
                        format_f(intensitat),
                        format_f(float(cable['seccio']),2),
                        format_f(capacitat),
                        propietari,
                        0
                    )
                    if actual == entregada:
                        estado = 0
                    else:
                        estado = 1
                else:
                    estado = 2

                output = [
                    'B{}'.format(linia['name']),
                    linia['cini'] or '',
                    origen or '',
                    final or '',
                    codi_ccuu or '',
                    comunitat,
                    comunitat,
                    format_f(100.0 - linia['perc_financament'], 3),
                    data_pm or '',
                    fecha_baja,
                    1,
                    1,
                    format_f(tensio, 3),
                    format_f(longitud, 3),
                    format_f(intensitat, 3),
                    format_f(cable['seccio'], 3),
                    format_f(capacitat, 3),
                    estado,
                    ct_name,
                    municipi,
                    zona,
                ]

                self.output_q.put(output)
                self.input_q.task_done()
            except Exception:
                self.input_q.task_done()
                traceback.print_exc()
                if self.raven:
                    self.raven.captureException()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import traceback

from libcnmc.core import MultiprocessBased
from libcnmc.res_4667.utils import get_resum_any_id
from libcnmc.utils import get_codigo_ccaa, get_name_ti, format_f


class LBT(MultiprocessBased):
    """
    Class to generate F2 of 4667
    """

    def __init__(self, **kwargs):
        """
        Class constructor
        :param kwargs: 
        """
        self.year = kwargs.pop("year")
        super(LBT, self).__init__(**kwargs)

    def get_sequence(self):
        """
        Generates the sequence of ids to make the report
        
        :return: List of ids
        :rtype: list
        """
        id_resum = get_resum_any_id(self.connection, self.year)
        search_macro = [("resums_inversio", "in", id_resum)]

        return self.connection.GiscedataCnmcLiniesbt.search(search_macro)

    def consumer(self):
        """
        Generates the line of the file
        :return: Line 
        :rtype: str
        """

        O = self.connection
        fields_to_read = [
            "cod_proyecto", "finalidad", "identificador_py", "cini_prv",
            "cod_tipo_inst", "codigo_ccaa_1", "codigo_ccaa_2", "anio_prev_aps",
            "longitud_prv", "capacidad_prv", "vol_inv_prev", "ayudas_prv",
            "financiacion_prv", "vpi_retribuible_prv", "estado",
            "actuacio_elegible_prtr"
        ]

        while True:
            try:
                item = self.input_q.get()
                self.progress_q.put(item)

                lbt = O.GiscedataCnmcLiniesbt.read(item, fields_to_read)
                output = [
                    lbt["cod_proyecto"][1],
                    lbt["finalidad"],
                    lbt["identificador_py"],
                    lbt["cini_prv"],
                    get_name_ti(O, lbt["cod_tipo_inst"][0]),
                    get_codigo_ccaa(O, lbt["codigo_ccaa_1"][0]),
                    get_codigo_ccaa(O, lbt["codigo_ccaa_2"][0]),
                    lbt["anio_prev_aps"],
                    format_f(lbt["longitud_prv"], 3) or "0.00",
                    lbt["capacidad_prv"],
                    format_f(lbt["vol_inv_prev"], 2) or "0.00",
                    format_f(lbt["ayudas_prv"], 2) or "0.00",
                    format_f(lbt["financiacion_prv"], 2) or "0.00",
                    format_f(lbt["vpi_retribuible_prv"], 2) or "0.00",
                    lbt["estado"],
                    lbt["actuacio_elegible_prtr"] or '',
                ]
                self.output_q.put(output)

            except Exception:
                traceback.print_exc()
                if self.raven:
                    self.raven.captureException()
            finally:
                self.input_q.task_done()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import absolute_import
from datetime import datetime
import traceback
from libcnmc.core import StopMultiprocessBased
from libcnmc.utils import (format_f, tallar_text, format_f_6181, get_codi_actuacio, convert_spanish_date,
                           get_forced_elements, adapt_diff)
from libcnmc.models import F1Res4666,F2Res4666

class FB1(StopMultiprocessBased):
    """
    Class that generates the FB1(1) file of  4666
    """
    def __init__(self, **kwargs):
        """
        Class constructor
        :param kwargs: year(generation year), codi_r1 R1 code
        :return: CT
        """
        super(FB1, self).__init__(**kwargs)
        self.extended = kwargs.get("extended", False)
        self.year = kwargs.pop('year', datetime.now().year - 1)
        self.codi_r1 = kwargs.pop('codi_r1')
        self.base_object = 'Linies AT'
        self.report_name = 'CNMC INVENTARI AT'
        self.layer = 'LBT\_%'
        self.embarrats = kwargs.pop('embarrats', False)
        self.linia_tram_include = {}
        self.forced_ids = {}
        self.prefix_AT = kwargs.pop('prefix_at', 'A') or 'A'
        self.prefix_BT = kwargs.pop('prefix_bt', 'B') or 'B'
        self.compare_field = '4666_entregada'
        self.dividir = kwargs.pop('div', False)

        O = self.connection

        id_res_like = self.connection.ResConfig.search(
            [('name', '=', 'giscegis_btlike_layer')])
        if id_res_like:
            self.layer = self.connection.ResConfig.read(
                id_res_like, ['value'])[0]['value']

        trafos_fields = O.GiscedataTransformadorTrafo.fields_get().keys()
        if 'node_id' in trafos_fields:
            ids_red = O.GiscedataTransformadorTrafo.search(
                [('reductor', '=', True)]
            )
            data_nodes = O.GiscedataTransformadorTrafo.read(
                ids_red, ["node_id"]
            )
            self.nodes_red = [
                nod["node_id"][1] for nod in data_nodes if nod["node_id"]
            ]
        else:
            ids_red = O.GiscegisBlocsTransformadorsReductors.search([])
            data_nodes = O.GiscegisBlocsTransformadorsReductors.read(
                ids_red, ["node"]
            )
            self.nodes_red = [
                nod["node"][1] for nod in data_nodes if nod["node"]
            ]

    def get_sequence(self):
        """
        Method that generates a list of ids to pass to the consummer

        :return: List of ids
        :rtype: list(int)
        """
        data_pm = '%s-01-01' % (self.year + 1)
        data_baixa = '%s-01-01' % self.year

        # AT
        search_params = [('criteri_regulatori', '!=', 'excloure'),
                         '|', ('data_pm', '=', False),
                              ('data_pm', '<', data_pm),
                         '|', ('data_baixa', '>=', data_baixa),
                              ('data_baixa', '=', False)]

        # Revisem que si està de baixa ha de tenir la data informada
        search_params += ['|',
                          '&', ('active', '=', False),
                               ('data_baixa', '!=', False),
                          ('active', '=', True)]

        obj_lat = self.connection.GiscedataAtTram
        ids = obj_lat.search(
            search_params, 0, 0, False, {'active_test': False})
        at_ids = list(set(ids))

        for elem in range(0, len(at_ids)):
            at_ids[elem] = 'at.{}'.format(at_ids[elem])

        # BT
        search_params = [('criteri_regulatori', '!=', 'excloure'),
                         '|', ('data_pm', '=', False),
                              ('data_pm', '<', data_pm),
                         '|', ('data_baixa', '>=', data_baixa),
                              ('data_baixa', '=', False)]

        # Revisem que si està de baixa ha de tenir la data informada
        search_params += ['|',
                          '&', ('active', '=', False),
                               ('data_baixa', '!=', False),
                          ('active', '=', True)]

        obj_lbt = self.connection.GiscedataBtElement
        ids = obj_lbt.search(
            search_params, 0, 0, False, {'active_test': False})
        bt_ids = list(set(ids))

        for elem in range(0, len(bt_ids)):
            bt_ids[elem] = 'bt.{}'.format(bt_ids[elem])

        at_bt_ids = at_ids + bt_ids
        return at_bt_ids

    def consumer(self):
        """
        Method that generates the csv file
        :return: List of arrays
        """
        O = self.connection

        fields_to_read = [
            'baixa', 'data_pm', 'data_industria', 'coeficient', 'cini', 'propietari', 'tensio_max_disseny_id', 'name',
            'origen', 'final', 'perc_financament', 'longitud_cad', 'cable', 'linia', 'model', 'model', 'punt_frontera',
            'tipus_instalacio_cnmc_id', 'data_baixa', 'baixa', 'longitud_cad', 'data_pm', 'circuits',
            'id_regulatori', self.compare_field, 'municipi',
        ]
        data_pm_limit = '{0}-01-01'.format(self.year + 1)

        def get_inst_name(element_id):
            vals = self.connection.GiscedataCts.read(
                element_id[0], ['name'])
            return vals['name']

        while True:
            try:
                item = self.input_q.get()
                if item == 'STOP':
                    self.input_q.task_done()
                    break
                self.progress_q.put(item)
                model = item.split('.')[0]
                item = int(item.split('.')[1])

                if model == 'at':

                    tram = O.GiscedataAtTram.read(item, fields_to_read)

                    # Coeficient per ajustar longituds de trams
                    coeficient = tram.get('coeficient', 1.0)
                    if tram.get('tipus_instalacio_cnmc_id', False):
                        id_ti = tram.get('tipus_instalacio_cnmc_id')[0]
                        codi_ccuu = O.GiscedataTipusInstallacio.read(
                            id_ti,
                            ['name'])['name']
                    else:
                        codi_ccuu = ''

                    # Comunitat
                    comunitat = ''
                    fun_ccaa = O.ResComunitat_autonoma.get_ccaa_from_municipi
                    if tram.get('municipi', False):
                        municipi_id = tram['municipi'][0]
                        id_comunitat = fun_ccaa(municipi_id)
                        comunitat_vals = O.ResComunitat_autonoma.read(id_comunitat[0], ['codi'])
                        if comunitat_vals:
                            comunitat = comunitat_vals['codi']

                    # Propietari
                    if tram.get('propietari', False):
                        if tram['propietari']:
                            propietari = '1'
                        else:
                            propietari = '0'
                    else:
                        propietari = '0'

                    # Tensión_explotación
                    tension_explotacion = ''
                    if tram.get('linia', False):
                        linia_id, linia_name = tram['linia']
                        if linia_id:
                            tensio_data = O.GiscedataAtLinia.read(linia_id, ['tensio_id'])
                            if tensio_data.get('tensio_id', False):
                                tension_explotacion = format_f(float(tensio_data['tensio_id'][1])/1000, 3)

                    # Tensión_construcción
                    tension_construccion = ''
                    if tram.get('tensio_max_disseny_id', False):
                        tension_construccion = format_f(float(tram['tensio_max_disseny_id'][1])/1000, 3)
                        if str(tension_construccion) == str(tension_explotacion):
                            tension_construccion = ''

                    # Longitud
                    if 'longitud_cad' in tram:
                        if self.dividir:
                            long_tmp = tram['longitud_cad']/tram.get(
                                'circuits', 1
                            ) or 1
                            longitud = round(
                                long_tmp * coeficient/1000.0, 3
                            ) or 0.001
                        else:
                            longitud = round(
                                tram['longitud_cad'] * coeficient/1000.0, 3
                            ) or 0.001
                    else:
                        longitud = 0

                    # Resistencia, Reactancia, Intensitat
                    resistencia, reactancia, intensitat = ['0,001', '0,001', '0,001']
                    if tram.get('cable', False):
                        cable_obj = O.GiscedataAtCables
                        cable_id = tram['cable'][0]
                        cable_data = cable_obj.read(cable_id, ['resistencia', 'reactancia', 'intensitat_admisible'])
                        if cable_data.get('resistencia', False):    
                            resistencia = format_f(
                                cable_data['resistencia'] * (float(tram['longitud_cad']) *
                                                             coeficient / 1000.0) or 0.0,
                                decimals=3)
                        if cable_data.get('reactancia', False):    
                            reactancia = format_f(
                                cable_data['reactancia'] * (float(tram['longitud_cad']) *
                                                            coeficient / 1000.0) or 0.0,
                                decimals=3)
                        if cable_data.get('resistencia', False):    
                            intensitat = format_f(
                                cable_data['intensitat_admisible'] or 0.0, decimals=3)

                    cable_data_val = [resistencia, reactancia, intensitat]
                    for index, val in enumerate(cable_data_val):
                        if val == '0,000':
                            cable_data_val[index] = '0,001'
                    resistencia, reactancia, intensitat = cable_data_val

                    # Modelo
                    if tram.get('model', False):
                        modelo = tram['model']
                    else:
                        modelo = ''

                    # Punt frontera
                    punt_frontera = '0'
                    if tram.get('punt_frontera', False):
                        punt_frontera = '1'

                    # Operación
                    operacion = '1'
                    if tram.get('operacion', False):
                        if tram['operacion']:
                            operacion = '1'
                        else:
                            operacion = '0'

                    # Causa baja
                    if tram.get('obra_id', False):
                        causa_baja = '0'
                        obra_id = tram['obra_id']
                        obra_at_obj = O.GiscedataProjecteObraTiAt
                        causa_baja_data = obra_at_obj.read(obra_id, ['causa_baja'])
                        if causa_baja_data.get('causa_baja', False):
                            causa_baja = causa_baja_data['causa_baja']
                    else:
                        causa_baja = '0'

                    # Fecha APS
                    if tram['data_pm']:
                        data_pm_linia = datetime.strptime(str(tram['data_pm']),
                                                          '%Y-%m-%d')
                        fecha_aps = data_pm_linia.strftime('%d/%m/%Y')


                    # OBRES

                    fields_to_read_obra = [
                        'name', 'cini', 'tipo_inversion', 'ccuu', 'codigo_ccaa', 'nivel_tension_explotacion',
                        'financiado', 'fecha_aps', 'fecha_baja', 'causa_baja', 'im_ingenieria', 'im_materiales',
                        'im_obracivil', 'im_trabajos', 'subvenciones_europeas', 'subvenciones_nacionales',
                        'subvenciones_prtr', 'avifauna', 'valor_auditado', 'valor_contabilidad', 'cuenta_contable',
                        'porcentaje_modificacion', 'motivacion', 'obra_id', 'identificador_baja'
                    ]

                    # OBRES
                    tram_obra = ''
                    obra_ti_at_obj = O.GiscedataProjecteObraTiAt
                    obra_ti_ids = obra_ti_at_obj.search([('element_ti_id', '=', tram['id'])])
                    if obra_ti_ids:
                        for obra_ti_id in obra_ti_ids:
                            obra_id_data = obra_ti_at_obj.read(obra_ti_id, ['obra_id'])
                            obra_id = obra_id_data['obra_id']
                            # Filtre d'obres finalitzades
                            data_finalitzacio_data = O.GiscedataProjecteObra.read(obra_id[0], ['data_finalitzacio'])
                            if data_finalitzacio_data:
                                if data_finalitzacio_data.get('data_finalitzacio', False):
                                    data_finalitzacio = data_finalitzacio_data['data_finalitzacio']

                                    inici_any = '{}-01-01'.format(self.year)
                                    fi_any = '{}-12-31'.format(self.year)
                                    if obra_id and data_finalitzacio and inici_any <= data_finalitzacio <= fi_any:
                                        tram_obra = O.GiscedataProjecteObraTiAt.read(obra_ti_id, fields_to_read_obra)
                            if tram_obra:
                                break

                    tipo_inversion = ''
                    # CAMPS OBRA
                    if tram_obra != '':
                        obra_year = data_finalitzacio.split('-')[0]
                        data_pm_year = fecha_aps.split('/')[2]
                        if tram_obra['tipo_inversion'] != '0' and obra_year != data_pm_year:
                            data_ip = convert_spanish_date(data_finalitzacio)
                        else:
                            data_ip = ''
                        if tram_obra.get('identificador_baja', False):
                            tram_id = tram_obra['identificador_baja'][0]
                            tram_name = O.GiscedataAtTram.read(tram_id, ['name'])['name']
                            identificador_baja = '{}{}'.format(self.prefix_AT, tram_name)
                        else:
                            identificador_baja = ''
                        tipo_inversion = tram_obra['tipo_inversion'] or ''
                        im_ingenieria = format_f_6181(tram_obra['im_ingenieria'] or 0.0, float_type='euro')
                        im_materiales = format_f_6181(tram_obra['im_materiales'] or 0.0, float_type='euro')
                        im_obracivil = format_f_6181(tram_obra['im_obracivil'] or 0.0, float_type='euro')
                        im_construccion = str(format_f(
                            float(im_materiales.replace(",", ".")) + float(im_obracivil.replace(",", "."))
                            , 2)).replace(".", ",")
                        im_trabajos = format_f_6181(tram_obra['im_trabajos'] or 0.0, float_type='euro')
                        subvenciones_europeas = format_f_6181(tram_obra['subvenciones_europeas'] or 0.0, float_type='euro')
                        subvenciones_nacionales = format_f_6181(tram_obra['subvenciones_nacionales'] or 0.0,
                                                                float_type='euro')
                        subvenciones_prtr = format_f_6181(tram_obra['subvenciones_prtr'] or 0.0, float_type='euro')
                        valor_auditado = format_f_6181(tram_obra['valor_auditado'] or 0.0, float_type='euro')
                        motivacion = get_codi_actuacio(O, tram_obra['motivacion'] and tram_obra['motivacion'][0]) if not \
                            tram_obra['fecha_baja'] else ''
                        cuenta_contable = tram_obra['cuenta_contable'] or ''
                        avifauna = int(tram_obra['avifauna'] == True)
                    else:
                        data_ip = ''
                        identificador_baja = ''
                        im_ingenieria = ''
                        im_construccion = ''
                        im_trabajos = ''
                        subvenciones_europeas = ''
                        subvenciones_nacionales = ''
                        subvenciones_prtr = ''
                        valor_auditado = ''
                        motivacion = ''
                        cuenta_contable = ''
                        avifauna = ''

                    # FINANCIADO
                    financiado = ''
                    if isinstance(tram.get('perc_financament', False), float):
                        financiado = 100 - tram['perc_financament']

                    # CAUSA_BAJA
                    causa_baja = '0'
                    if tram_obra:
                        if tram_obra.get('causa_baja', False):
                            causa_baja = tram_obra['causa_baja']

                    # Node inicial / Node final
                    o_nivell_tensio = ''
                    if tram.get('tensio_max_disseny_id', False):
                        nivell_tensio_id = tram['tensio_max_disseny_id'][0]
                        o_nivell_tensio = O.GiscedataTensionsTensio.read(nivell_tensio_id, ["tensio"])["tensio"]
                    else:
                        if tram.get('linia', False):
                            linia_id, linia_name = tram['linia']
                            tensio_data = O.GiscedataAtLinia.read(linia_id, ['tensio_id'])
                            if tensio_data.get('tensio_id', False):
                                o_nivell_tensio = float(tensio_data['tensio_id'][1])
                    if o_nivell_tensio:
                        o_nivell_tensio = format_f(float(o_nivell_tensio) / 1000.0, 3)

                    # identificador_tramo
                    if tram.get('id_regulatori', False):
                        o_tram = tram['id_regulatori']
                    else:
                        o_tram = '{}{}'.format(self.prefix_AT, tram['name'])

                    if 'edge_id' in O.GiscedataAtTram.fields_get().keys():
                        tram_edge = O.GiscedataAtTram.read(
                            tram['id'], ['edge_id']
                        )['edge_id']
                        if not tram_edge:
                            res = O.GiscegisEdge.search(
                                [
                                    ('id_linktemplate', '=', tram['name']),
                                    ('layer', 'not ilike', self.layer),
                                    ('layer', 'not ilike', 'EMBARRA%BT%')
                                ]
                            )
                            if not res or len(res) > 1:
                                edge = {
                                    'start_node': (0, '%s_0' % tram['name']),
                                    'end_node': (0, '%s_1' % tram['name'])
                                }
                            else:
                                edge = O.GiscegisEdge.read(
                                    res[0], ['start_node', 'end_node']
                                )
                        else:
                            edge = O.GiscegisEdge.read(
                                tram_edge[0], ['start_node', 'end_node']
                            )
                    else:
                        res = O.GiscegisEdge.search(
                            [
                                ('id_linktemplate', '=', tram['name']),
                                ('layer', 'not ilike', self.layer),
                                ('layer', 'not ilike', 'EMBARRA%BT%')
                            ]
                        )
                        if not res or len(res) > 1:
                            edge = {
                                'start_node': (0, '%s_0' % tram['name']),
                                'end_node': (0, '%s_1' % tram['name'])
                            }
                        else:
                            edge = O.GiscegisEdge.read(
                                res[0], ['start_node', 'end_node']
                            )

                    node_inicial = tallar_text(edge['start_node'][1], 20)
                    node_inicial = node_inicial.replace('*', '')
                    if node_inicial in self.nodes_red:
                        node_inicial = "{}-{}".format(
                            node_inicial, o_nivell_tensio
                        )
                    node_final = tallar_text(edge['end_node'][1], 20)
                    node_final = node_final.replace('*', '')
                    if node_final in self.nodes_red:
                        node_final = "{}-{}".format(
                            node_final, o_nivell_tensio
                        )

                    if tram.get('data_baixa'):
                        if tram.get('data_baixa') > data_pm_limit:
                            fecha_baja = ''
                        else:
                            tmp_date = datetime.strptime(
                                tram.get('data_baixa'), '%Y-%m-%d')
                            fecha_baja = tmp_date.strftime('%d/%m/%Y')
                    else:
                        fecha_baja = ''

                    # Fecha APS / Estado
                    if modelo == 'M':
                        estado = ''
                        fecha_aps = ''
                        operacion = 0
                    else:
                        # Estado
                        if tram[self.compare_field]:
                            data_entregada = tram[self.compare_field]
                            entregada = F1Res4666(**data_entregada)
                            actual = F1Res4666(
                                o_tram,
                                tram['cini'],
                                node_inicial or edge['start_node'][1],
                                node_final or edge['end_node'][1],
                                codi_ccuu,
                                comunitat,
                                comunitat,
                                '',
                                fecha_aps,
                                fecha_baja,
                                tram.get('circuits', 1) or 1,
                                1,
                                tension_explotacion,
                                format_f(longitud, 3),
                                intensitat,
                                '',
                                '',
                                0
                            )
                            if actual == entregada and fecha_baja == '':
                                estado = '0'
                                if tram_obra:
                                    estado = '1'
                            else:
                                self.output_m.put("{} {}".format(tram["name"], adapt_diff(actual.diff(entregada))))
                                estado = 1
                        else:
                            estado = '2'

                    if fecha_baja:
                        motivacion = ''
                        tipo_inversion = ''

                    # L'any 2022 no es declaren subvencions PRTR
                    subvenciones_prtr = ''

                    if causa_baja == '0':
                        fecha_baja = ''

                    if modelo == 'E':
                        tipo_inversion = '0'
                        estado = '1'

                    output = [
                        o_tram,  # IDENTIFICADOR
                        tram.get('cini', '') or '',         # CINI
                        codi_ccuu or '',                    # CODIGO_CCUU
                        node_inicial or edge['start_node'][1],    # ORIGEN
                        node_final or edge['end_node'][1],       # DESTINO
                        comunitat,                          # CODIGO_CCAA_1
                        comunitat,                          # CODIGO_CCAA_2
                        propietari,                         # PROPIEDAD
                        tension_explotacion,                # TENSION_EXPLOTACIÓN
                        tension_construccion,               # TENSION_CONSTRUCCIÓN
                        format_f(longitud, 3),              # LONGITUD
                        resistencia,                        # RESISTENCIA
                        reactancia,                         # REACTANCIA
                        intensitat,                         # INTENSIDAD
                        estado,                             # ESTADO
                        punt_frontera,                      # PUNTO_FRONTERA
                        modelo,                             # MODELO
                        operacion,                          # OPERACIÓN
                        fecha_aps,                          # FECHA APS
                        causa_baja,                         # CAUSA BAJA
                        fecha_baja or '',                   # FECHA BAJA
                        data_ip,                            # FECHA IP
                        tipo_inversion,                     # TIPO INVERSION
                        motivacion,                         # MOTIVACION
                        im_ingenieria,                      # IM_TRAMITES
                        im_construccion,                    # IM_CONSTRUCCION
                        im_trabajos,                        # IM_TRABAJOS
                        valor_auditado,  # VALOR AUDITADO
                        format_f(financiado, decimals=2),   # FINANCIADO
                        subvenciones_europeas,              # SUBVENCIONES EUROPEAS
                        subvenciones_nacionales,            # SUBVENCIONES NACIONALES
                        subvenciones_prtr,                  # SUBVENCIONES PRTR
                        cuenta_contable,                    # CUENTA CONTABLE
                        avifauna,                           # AVIFAUNA
                        identificador_baja,                 # ID_BAJA
                    ]

                    self.output_q.put(output)

                if model == 'bt':
                    fields_to_read = ['name', 'cini', 'coeficient', 'municipi', 'voltatge', 'tensio_const',
                                      'coeficient', 'longitud_cad', 'punt_frontera', 'model', 'operacion',
                                      'propietari', 'edge_id', 'cable', 'tipus_instalacio_cnmc_id',
                                      'data_pm', 'data_baixa', 'id_regulatori', self.compare_field,
                                      'perc_financament',
                                      ]

                    linia = O.GiscedataBtElement.read(item, fields_to_read)

                    # Identificador_tramo
                    if linia.get('id_regulatori', False):
                        identificador_tramo = linia['id_regulatori']
                    else:
                        identificador_tramo = '{}{}'.format(self.prefix_BT, linia['name'])

                    # CINI
                    cini = ''
                    if linia.get('cini', False):
                        cini = linia['cini']

                    # CODIGO CCUU
                    if linia.get('tipus_instalacio_cnmc_id', False):
                        id_ti = linia['tipus_instalacio_cnmc_id'][0]
                        codigo_ccuu = O.GiscedataTipusInstallacio.read(
                            id_ti,
                            ['name'])['name']
                    else:
                        codigo_ccuu = ''

                    # NUDO INICIAL / NUDO FINAL
                    if 'edge_id' in O.GiscedataBtElement.fields_get().keys():
                        bt_edge = O.GiscedataBtElement.read(
                            linia['id'], ['edge_id']
                        )['edge_id']
                        if not bt_edge:
                            res = O.GiscegisEdge.search(
                                [('id_linktemplate', '=', linia['name']),
                                 '|',
                                 ('layer', 'ilike', self.layer),
                                 ('layer', 'ilike', 'EMBARRA%BT%')
                                 ])
                            if not res or len(res) > 1:
                                edge = {'start_node': (0, '%s_0' % linia['name']),
                                        'end_node': (0, '%s_1' % linia['name'])}
                            else:
                                edge = O.GiscegisEdge.read(
                                    res[0], ['start_node', 'end_node']
                                )
                        else:
                            edge = O.GiscegisEdge.read(
                                bt_edge[0], ['start_node', 'end_node']
                            )
                    else:
                        res = O.GiscegisEdge.search(
                            [('id_linktemplate', '=', linia['name']),
                             '|',
                             ('layer', 'ilike', self.layer),
                             ('layer', 'ilike', 'EMBARRA%BT%')
                             ])
                        if not res or len(res) > 1:
                            edge = {'start_node': (0, '%s_0' % linia['name']),
                                    'end_node': (0, '%s_1' % linia['name'])}
                        else:
                            edge = O.GiscegisEdge.read(res[0], ['start_nOde',
                                                                'end_node'])
                    node_inicial = tallar_text(edge['start_node'][1], 20)
                    node_inicial = node_inicial.replace('*', '')
                    node_final = tallar_text(edge['end_node'][1], 20)
                    node_final = node_final.replace('*', '')

                    # CCAA 1 / CCAA 2
                    ccaa_1 = ccaa_2 = ''
                    if linia['municipi']:
                        ccaa_obj = O.ResComunitat_autonoma
                        id_comunitat = ccaa_obj.get_ccaa_from_municipi(
                            linia['municipi'][0])
                        id_comunitat = id_comunitat[0]
                        comunidad = ccaa_obj.read(id_comunitat, ['codi'])
                        if comunidad:
                            ccaa_1 = ccaa_2 = comunidad['codi']

                    # PROPIEDAD
                    propiedad = '0'
                    if linia.get('propietari', False):
                        if linia['propietari']:
                            propiedad = '1'
                        else:
                            propiedad = '0'

                    # TENSION EXPLOTACION
                    tension_explotacion = ''
                    if linia.get('voltatge', False):
                        tension_explotacion = format_f(float(linia['voltatge'])/1000, 3)

                    # TENSION CONSTRUCCION
                    tension_construccion = ''
                    if linia.get('tensio_const', False):
                        tension_construccion = format_f(float(linia['tensio_max_disseny_id'][1])/1000, 3)
                        if str(tension_construccion) == str(tension_explotacion):
                            tension_construccion = ''

                    # LONGITUD
                    coeficient = linia.get('coeficient', 1.0)
                    if 'longitud_cad' in linia:
                        if self.dividir:
                            long_tmp = linia['longitud_cad']/linia.get(
                                'circuits', 1
                            ) or 1
                            longitud = round(
                                long_tmp * coeficient/1000.0, 3
                            ) or 0.001
                        else:
                            longitud = round(
                                linia['longitud_cad'] * coeficient/1000.0, 3
                            ) or 0.001
                    else:
                        longitud = 0

                    # RESISTENCIA, REACTANCIA, INTENSITAT
                    resistencia, reactancia, intensitat = ['0,001', '0,001', '0,001']
                    if linia.get('cable', False):
                        cable_obj = O.GiscedataBtCables
                        cable_id = linia['cable'][0]
                        cable_data = cable_obj.read(cable_id, ['resistencia', 'reactancia', 'intensitat_admisible'])
                        longitud = format_f(
                            round(float(linia['longitud_cad']) *
                                  coeficient / 1000.0, 3) or 0.001, decimals=3)
                        if cable_data.get('resistencia', False):    
                            resistencia = format_f(
                                cable_data['resistencia'] * (float(linia['longitud_cad']) *
                                                             coeficient / 1000.0) or 0.0,
                                decimals=3)
                        if cable_data.get('reactancia', False):    
                            reactancia = format_f(
                                cable_data['reactancia'] * (float(linia['longitud_cad']) *
                                                            coeficient / 1000.0) or 0.0,
                                decimals=3)
                        if cable_data.get('resistencia', False):    
                            intensitat = format_f(
                                cable_data['intensitat_admisible'] or 0.0, decimals=3)

                    cable_data_val = [resistencia, reactancia, intensitat]
                    for index, val in enumerate(cable_data_val):
                        if val == '0,000':
                            cable_data_val[index] = '0,001'
                    resistencia, reactancia, intensitat = cable_data_val

                    # PUNTO FRONTERA
                    punto_frontera = '0'
                    if linia.get('punt_frontera', False):
                        punto_frontera = '1'

                    # OPERACION
                    operacion = '1'
                    if linia.get('operacion', False):
                        if linia['operacion']:
                            operacion = '1'
                        else:    
                            operacion = '0'

                    # FECHA BAJA
                    if linia.get('data_baixa'):
                        if linia.get('data_baixa') > data_pm_limit:
                            fecha_baja = ''
                        else:
                            tmp_date = datetime.strptime(
                                linia.get('data_baixa'), '%Y-%m-%d')
                            fecha_baja = tmp_date.strftime('%d/%m/%Y')
                    else:
                        fecha_baja = ''

                    # Fecha APS
                    if linia['data_pm']:
                        data_pm_linia = datetime.strptime(str(linia['data_pm']),
                                                          '%Y-%m-%d')
                        fecha_aps = data_pm_linia.strftime('%d/%m/%Y')

                    # OBRES
                    fields_to_read_obra = [
                        'name', 'cini', 'tipo_inversion', 'ccuu', 'codigo_ccaa', 'nivel_tension_explotacion',
                        'financiado', 'fecha_aps', 'fecha_baja', 'causa_baja', 'im_ingenieria', 'im_materiales',
                        'im_obracivil', 'im_trabajos', 'subvenciones_europeas', 'subvenciones_nacionales',
                        'subvenciones_prtr', 'avifauna', 'valor_auditado', 'valor_contabilidad', 'cuenta_contable',
                        'porcentaje_modificacion', 'motivacion', 'obra_id', 'identificador_baja'
                    ]

                    linia_obra = ''
                    obra_ti_bt_obj = O.GiscedataProjecteObraTiBt
                    obra_ti_ids = obra_ti_bt_obj.search([('element_ti_id', '=', linia['id'])])
                    if obra_ti_ids:
                        for obra_ti_id in obra_ti_ids:
                            obra_id_data = obra_ti_bt_obj.read(obra_ti_id, ['obra_id'])
                            obra_id = obra_id_data['obra_id']
                            # Filtre d'obres finalitzades
                            data_finalitzacio_data = O.GiscedataProjecteObra.read(obra_id[0], ['data_finalitzacio'])
                            if data_finalitzacio_data:
                                if data_finalitzacio_data.get('data_finalitzacio', False):
                                    data_finalitzacio = data_finalitzacio_data['data_finalitzacio']

                                    inici_any = '{}-01-01'.format(self.year)
                                    fi_any = '{}-12-31'.format(self.year)
                                    if obra_id and data_finalitzacio and inici_any <= data_finalitzacio <= fi_any:
                                        linia_obra = O.GiscedataProjecteObraTiBt.read(obra_ti_id, fields_to_read_obra)
                            if linia_obra:
                                break

                    tipo_inversion = ''
                    # CAMPS OBRA
                    if linia_obra != '':
                        obra_year = data_finalitzacio.split('-')[0]
                        data_pm_year = fecha_aps.split('/')[2]
                        if linia_obra['tipo_inversion'] != '0' and obra_year != data_pm_year:
                            data_ip = convert_spanish_date(data_finalitzacio)
                        else:
                            data_ip = ''
                        if linia_obra.get('identificador_baja', False):
                            elem_id = linia_obra['identificador_baja'][0]
                            elem_name = O.GiscedataBtElement.read(elem_id, ['name'])['name']
                            identificador_baja = '{}{}'.format(self.prefix_BT, elem_name)
                        else:
                            identificador_baja = ''
                        tipo_inversion = linia_obra['tipo_inversion'] or ''
                        im_ingenieria = format_f_6181(linia_obra['im_ingenieria'] or 0.0, float_type='euro')
                        im_materiales = format_f_6181(linia_obra['im_materiales'] or 0.0, float_type='euro')
                        im_obracivil = format_f_6181(linia_obra['im_obracivil'] or 0.0, float_type='euro')
                        im_construccion = str(format_f(
                            float(im_materiales.replace(",", ".")) + float(im_obracivil.replace(",", "."))
                            , 2)).replace(".", ",")
                        im_trabajos = format_f_6181(linia_obra['im_trabajos'] or 0.0, float_type='euro')
                        subvenciones_europeas = format_f_6181(linia_obra['subvenciones_europeas'] or 0.0,
                                                              float_type='euro')
                        subvenciones_nacionales = format_f_6181(linia_obra['subvenciones_nacionales'] or 0.0,
                                                                float_type='euro')
                        subvenciones_prtr = format_f_6181(linia_obra['subvenciones_prtr'] or 0.0, float_type='euro')
                        valor_auditado = format_f_6181(linia_obra['valor_auditado'] or 0.0, float_type='euro')
                        motivacion = get_codi_actuacio(O, linia_obra['motivacion'] and linia_obra['motivacion'][0]) if not \
                            linia_obra['fecha_baja'] else ''
                        cuenta_contable = linia_obra['cuenta_contable'] or ''
                        avifauna = int(linia_obra['avifauna'] == True)
                    else:
                        data_ip = ''
                        identificador_baja = ''
                        im_ingenieria = ''
                        im_construccion = ''
                        im_trabajos = ''
                        subvenciones_europeas = ''
                        subvenciones_nacionales = ''
                        subvenciones_prtr = ''
                        valor_auditado = ''
                        motivacion = ''
                        cuenta_contable = ''
                        avifauna = ''

                    # FINANCIADO
                    financiado = ''
                    if isinstance(linia.get('perc_financament', False), float):
                        financiado = 100 - linia['perc_financament']

                    # CAUSA_BAJA
                    causa_baja = '0'
                    if linia_obra:
                        if linia_obra.get('causa_baja', False):
                            causa_baja = linia_obra['causa_baja']

                    # MODELO
                    modelo = ''
                    if linia.get('model', False):
                        modelo = linia['model']

                    # Fecha APS / Estado
                    if modelo == 'M':
                        estado = ''
                        fecha_aps = ''
                        operacion = 0
                    else:
                        # Estado
                        if linia[self.compare_field]:
                            data_entregada = linia[self.compare_field]
                            entregada = F2Res4666(**data_entregada)
                            actual = F2Res4666(
                                identificador_tramo,
                                linia['cini'],
                                node_inicial,
                                node_final,
                                codigo_ccuu,
                                ccaa_1,
                                ccaa_2,
                                '',
                                fecha_aps,
                                fecha_baja,
                                1,
                                1,
                                tension_explotacion,
                                format_f(longitud, 3),
                                intensitat,
                                '',
                                '',
                                0
                            )
                            if actual == entregada and fecha_baja == '':
                                estado = '0'
                                if linia_obra:
                                    estado = '1'
                            else:
                                self.output_m.put("{} {}".format(linia["name"], adapt_diff(actual.diff(entregada))))
                                estado = '1'
                        else:
                            estado = '2'

                    if fecha_baja:
                        motivacion = ''
                        tipo_inversion = ''

                    # L'any 2022 no es declaren subvencions PRTR
                    subvenciones_prtr = ''

                    if causa_baja == '0':
                        fecha_baja = ''

                    if modelo == 'E':
                        tipo_inversion = '0'
                        estado = '1'

                    output = [
                        identificador_tramo,  # IDENTIFICADOR TRAMO
                        linia.get('cini', '') or '',  # CINI
                        codigo_ccuu or '',  # CODIGO_CCUU
                        node_inicial,  # ORIGEN
                        node_final,  # DESTINO
                        ccaa_1,  # CODIGO_CCAA_1
                        ccaa_2,  # CODIGO_CCAA_2
                        propiedad,  # PROPIEDAD
                        tension_explotacion,  # TENSION_EXPLOTACIÓN
                        tension_construccion,  # TENSION_CONSTRUCCIÓN
                        format_f(longitud, 3),  # LONGITUD
                        resistencia,  # RESISTENCIA
                        reactancia,  # REACTANCIA
                        intensitat,  # INTENSIDAD
                        estado,  # ESTADO
                        punto_frontera,  # PUNTO_FRONTERA
                        modelo,  # MODELO
                        operacion,  # OPERACIÓN
                        fecha_aps,  # FECHA APS
                        causa_baja,  # CAUSA BAJA
                        fecha_baja or '',  # FECHA BAJA
                        data_ip,  # FECHA IP
                        tipo_inversion,  # TIPO INVERSION
                        motivacion,  # MOTIVACION
                        im_ingenieria,  # IM_TRAMITES
                        im_construccion,  # IM_CONSTRUCCION
                        im_trabajos,  # IM_TRABAJOS
                        valor_auditado,  # VALOR AUDITADO
                        format_f(financiado, decimals=2),  # FINANCIADO
                        subvenciones_europeas,  # SUBVENCIONES EUROPEAS
                        subvenciones_nacionales,  # SUBVENCIONES NACIONALES
                        subvenciones_prtr,  # SUBVENCIONES PRTR
                        cuenta_contable,  # CUENTA CONTABLE
                        avifauna,  # AVIFAUNA
                        identificador_baja,  # ID_BAJA
                    ]

                    self.output_q.put(output)
                self.input_q.task_done()
            except Exception:
                self.input_q.task_done()
                traceback.print_exc()
                if self.raven:
                    self.raven.captureException()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
INVENTARI DE CNMC Condensadors
"""
from __future__ import absolute_import
import sys
from datetime import datetime
import traceback
from operator import itemgetter

from libcnmc.core import StopMultiprocessBased
from libcnmc.utils import format_f, get_forced_elements


class CON(StopMultiprocessBased):
    """
    Class that generates the Maquinas/Condensadores(5) file of the 4666
    """
    def __init__(self, **kwargs):
        """
        Class constructor
        :param kwargs: year(generation year), codi_r1 R1 code
        :return: CT
        """
        super(CON, self).__init__(**kwargs)
        self.year = kwargs.pop('year', datetime.now().year - 1)
        self.codi_r1 = kwargs.pop('codi_r1')
        self.base_object = 'Línies CON'

        tension_fields_to_read = ['l_inferior', 'l_superior', 'tensio']
        tension_vals = self.connection.GiscedataTensionsTensio.read(
            self.connection.GiscedataTensionsTensio.search([]),
            tension_fields_to_read)

        self.tension_norm = [(t['l_inferior'], t['l_superior'], t['tensio'])
                             for t in tension_vals]
        t_norm_txt = ''
        for t in sorted(self.tension_norm, key=itemgetter(2)):
            t_norm_txt += '[{0:6d} <= {2:6d} < {1:6d}]\n'.format(*t)
        sys.stderr.write('Tensions normalitzades: \n{0}'.format(t_norm_txt))
        sys.stderr.flush()
        self.report_name = 'CNMC INVENTARI CON'

    def get_sequence(self):
        """
        Method that generates a list of ids to pass to the consummer

        :return: List of ids to pass to the consumer
        :rtype: list[int]
        """
        data_pm = '{0}-01-01'.format(self.year + 1)
        data_baixa = '{0}-01-01'.format(self.year)
        search_params = [('propietari', '=', True),
                         '|', ('data_pm', '=', False),
                         ('data_pm', '<', data_pm),
                         '|', ('data_baixa', '>', data_baixa),
                         ('data_baixa', '=', False)
                         ]
        # Revisem que si està de baixa ha de tenir la data informada.
        search_params += ['|',
                          '&', ('active', '=', False),
                          ('data_baixa', '!=', False),
                          ('active', '=', True)]

        forced_ids = get_forced_elements(
            self.connection,
            "giscedata.condensadors"
        )

        ids_condensadors = self.connection.GiscedataCondensadors.search(
            search_params, 0, 0, False, {'active_test': False})

        ids_condensadors = ids_condensadors + forced_ids["include"]
        ids_condensadors = list(set(ids_condensadors) - set(forced_ids["exclude"]))

        return list(set(ids_condensadors))

    def get_norm_tension(self, tension):
        """
        Method that gives the tension normalizada
        :param tension: tension as int
        :return: Tension normalizada
        """
        if not tension:
            return tension

        for t in self.tension_norm:
            if t[0] <= tension < t[1]:
                return t[2]

        sys.stderr.write('WARN: Tensió inexistent: {0}\n'.format(tension))
        sys.stderr.flush()
        return tension

    def consumer(self):
        """
        Method that generates the csv file
        :return: List of arrays
        """
        O = self.connection
        fields_to_read = [
            'cini', 'data_pm', 'ct_id', 'name', 'potencia_nominal', 'tensio_id',
            'numero_fabricacio', 'participacio', 'potencia_instalada',
            'tipus_instalacio_cnmc_id', 'conexions', 'data_baixa']

        data_pm_limit = '{0}-01-01'.format(self.year + 1)
        data_baixa_limit = '{0}-01-01'.format(self.year)
        while True:
            try:
                item = self.input_q.get()
                if item == "STOP":
                    self.input_q.task_done()
                    break
                self.progress_q.put(item)

                cond = O.GiscedataCondensadors.read(item, fields_to_read)
                codigo_ccuu = O.GiscedataTipusInstallacio.read(
                    cond['tipus_instalacio_cnmc_id'][0], ['name'])['name']

                data_pm = ''
                if cond['data_pm']:
                    data_pm = datetime.strptime(
                        str(cond['data_pm']), '%Y-%m-%d')
                    data_pm = data_pm.strftime('%d/%m/%Y')

                comunitat = ''
                financiacio = round(100.0 - float(cond['participacio']), 2)

                id_municipi = ''
                sys.stderr.write('CT {0} -> '.format(cond['ct_id']))
                denominacio = cond['ct_id']
                if cond['ct_id']:
                    cts = O.GiscedataCts.read(cond['ct_id'][0],
                                              ['id_municipi', 'descripcio'])
                    if cts['id_municipi']:
                        id_municipi = cts['id_municipi'][0]
                    denominacio = cts['descripcio']

                if id_municipi:
                    fun_ccaa = O.ResComunitat_autonoma.get_ccaa_from_municipi
                    id_comunitat = fun_ccaa(id_municipi)
                    comunidad = O.ResComunitat_autonoma.read(
                        id_comunitat, ['codi'])
                    comunitat = comunidad[0]['codi']
                tensio_primari = O.GiscedataTensionsTensio.read(
                    cond['tensio_id'][0], ['tensio'])['tensio']
                tensio_secundari = tensio_primari
                if cond['data_baixa']:
                    if cond['data_baixa'] < data_pm_limit:
                        tmp_date = datetime.strptime(
                            cond['data_baixa'], '%Y-%m-%d')
                        fecha_baja = tmp_date.strftime('%d/%m/%Y')
                    else:
                        fecha_baja = ''
                else:
                    fecha_baja = ''
                if cond['data_pm'] > data_baixa_limit:
                    estado = '2'
                elif fecha_baja != '':
                    estado = '1'
                else:
                    estado = '0'
                capacidad = format_f(cond['potencia_instalada']/1000, 3)
                output = [
                    '{0}'.format(cond['name']),
                    cond['cini'] or '',
                    denominacio or '',
                    codigo_ccuu,
                    comunitat or '',
                    format_f(float(tensio_primari)/1000.0),
                    format_f(float(tensio_secundari)/1000.0),
                    format_f(financiacio, 2),
                    data_pm,
                    fecha_baja,
                    capacidad,
                    estado
                ]

                self.output_q.put(output)
                self.input_q.task_done()
            except Exception:
                self.input_q.task_done()
                traceback.print_exc()
                if self.raven:
                    self.raven.captureException()



# -*- coding: utf-8 -*-
from libcnmc.core import StopMultiprocessBased
from libcnmc.utils import format_f
from datetime import datetime
import traceback


class FC6(StopMultiprocessBased):

    def __init__(self, **kwargs):
        super(FC6, self).__init__(**kwargs)
        self.year = kwargs.pop('year', datetime.now().year - 1)
        self.codi_r1 = kwargs.pop('codi_r1')
        self.report_name = (
            "Formulario C6: Información relativa a activos enajenados"
        )
        self.base_object = 'Formulario C6'

    def get_sequence(self):
        c6_ids = self.connection.model('cir8.2021.c6').search(
            [('year', '=', self.year)]
        )
        return c6_ids

    def consumer(self):
        O = self.connection
        fields_to_read = [
            'cod_operacion', 'tipo_operacion', 'coco', 'descripcion',
            'fecha_inicio', 'fecha_fin', 'ingreso_contrato', 'ingreso',
            'inmovilizado', 'gasto'
        ]
        while True:
            try:
                item = self.input_q.get()
                if item == 'STOP':
                    self.input_q.task_done()
                    break
                self.progress_q.put(item)

                c6 = O.model('cir8.2021.c6').read(item, fields_to_read)

                # CÓDIGO OPERACION
                o_cod_operacion = ''
                if c6.get('cod_operacion', False):
                    o_cod_operacion = c6['cod_operacion']

                # TIPO OPERACION
                o_tipo_operacion = ''
                if c6.get('tipo_operacion', False):
                    o_tipo_operacion = c6['tipo_operacion']

                # COCO
                o_coco = ''
                if c6.get('coco', False):
                    o_coco = c6['coco']

                # DESCRIPCION
                o_descripcion = ''
                if c6.get('descripcion', False):
                    o_descripcion = c6['descripcion']

                # FECHA INICIO
                o_fecha_inicio = '00/00/0000'
                if c6.get('fecha_inicio', False):
                    fecha_inicio_p = datetime.strptime(
                        str(c6['fecha_inicio']),
                        '%Y-%m-%d')
                    o_fecha_inicio = fecha_inicio_p.strftime('%d/%m/%Y')

                # FECHA FIN
                o_fecha_fin = ''
                if c6.get('fecha_fin', False):
                    fecha_fin_p = datetime.strptime(
                        str(c6['fecha_fin']),
                        '%Y-%m-%d')
                    o_fecha_fin = fecha_fin_p.strftime('%d/%m/%Y')

                # INGRESO CONTRATO
                o_ingreso_contrato = ''
                if c6.get('ingreso_contrato', False):
                    o_ingreso_contrato = format_f(
                        c6['ingreso_contrato'], decimals=2
                    )

                # INGRESO
                o_ingreso = ''
                if c6.get('ingreso', False):
                    o_ingreso = format_f(c6['ingreso'], decimals=2)

                # INMOVILIZADO
                o_inmovilizado = ''
                if c6.get('inmovilizado', False):
                    o_inmovilizado = format_f(c6['inmovilizado'], decimals=2)

                # GASTO
                o_gasto = ''
                if c6.get('gasto', False):
                    o_gasto = format_f(c6['gasto'], decimals=2)

                self.output_q.put([
                    o_cod_operacion,            # CÓDIGO OPERACION
                    o_tipo_operacion,           # TIPO OPERACION
                    o_coco,                     # COCO
                    o_descripcion,              # DESCRIPCION
                    o_fecha_inicio,             # FECHA INICIO
                    o_fecha_fin,                # FECHA FIN
                    o_ingreso_contrato,         # INGRESO CONTRATO
                    o_ingreso,                  # INGRESO
                    o_inmovilizado,             # INMOVILIZADO
                    o_gasto,                    # GASTO
                ])
                self.input_q.task_done()
            except Exception:
                self.input_q.task_done()
                traceback.print_exc()
                if self.raven:
                    self.raven.captureException()



from FA1 import FA1
from FA2 import FA2
from FA3 import FA3
from FA4 import FA4
from FA5 import FA5
from FB1 import FB1
from FB1_1 import FB1_1
from FB2 import FB2
from FB2_1 import FB2_1
from FB2_2 import FB2_2
from FB3 import FB3
from FB3_1 import FB3_1
from FB3_2 import FB3_2
from FB3_3 import FB3_3
from FB4 import FB4
from FB5 import FB5
from FB6 import FB6
from FB7 import FB7
from FB8 import FB8
from FB9 import FB9
from FC1 import FC1
from FC2 import FC2
from FC3 import FC3
from FC4 import FC4
from FC5 import FC5
from FC7 import FC7
from FD1 import FD1
from FD2 import FD2
from FC6 import FC6
from FF1 import FF1



from f1_4131 import F1Res4131
from f2_4131 import F2Res4131
from f3_4131 import F3Res4131
from f4_4131 import F4Res4131
from f5_4131 import F5Res4131
from f6_4131 import F6Res4131
from f7_4131 import F7Res4131
from f8_4131 import F8Res4131



# -*- coding: utf-8 -*-
from datetime import datetime
import traceback
from libcnmc.core import StopMultiprocessBased
from libcnmc.utils import convert_srid, get_srid, format_f

class FB1_1(StopMultiprocessBased):
    def __init__(self, **kwargs):
        super(FB1_1, self).__init__(**kwargs)
        self.codi_r1 = kwargs.pop('codi_r1')
        self.year = kwargs.pop('year', datetime.now().year - 1)
        self.report_name = 'FB1.1 - Topología real'
        self.base_object = 'Topología'
        self.prefix_AT = kwargs.pop('prefix_at', 'A') or 'A'
        self.prefix_BT = kwargs.pop('prefix_bt', 'B') or 'B'

    def get_sequence(self):
        data_pm = '%s-01-01' % (self.year + 1)
        data_baixa = '%s-01-01' % self.year
        search_params = [('criteri_regulatori', '!=', 'excloure'),
                         '|', ('data_pm', '=', False),
                         ('data_pm', '<', data_pm),
                         '|', ('data_baixa', '>=', data_baixa),
                         ('data_baixa', '=', False),
                         ]
        # Revisem que si està de baixa ha de tenir la data informada.
        search_params += ['|',
                          '&', ('active', '=', False),
                          ('data_baixa', '!=', False),
                          ('active', '=', True)]
        res_at = self.connection.GiscedataAtTram.search(
            search_params, 0, 0, False, {'active_test': False})
        res_bt = self.connection.GiscedataBtElement.search(
            search_params, 0, 0, False, {'active_test': False})
        at = [(r, 'at') for r in res_at]
        bt = [(r, 'bt') for r in res_bt]
        return at + bt

    def consumer(self):
        o = self.connection
        fields_to_read = [
            'id', 'name', 'geom', 'id_regulatori'
        ]
        while True:
            try:
                # generar tramos
                item = self.input_q.get()
                if item == 'STOP':
                    self.input_q.task_done()
                    break
                self.progress_q.put(item)
                if item[1] == 'at':
                    tramo = o.GiscedataAtTram.read(item[0], fields_to_read)

                    # identificador_tramo
                    if tramo.get('id_regulatori', False):
                        o_tram = tramo['id_regulatori']
                    else:
                        o_tram = '{}{}'.format(self.prefix_AT, tramo['name'])

                elif item[1] == 'bt':
                    tramo = o.GiscedataBtElement.read(item[0], fields_to_read)

                    # identificador_tramo
                    if tramo.get('id_regulatori', False):
                        o_tram = tramo['id_regulatori']
                    else:
                        o_tram = '{}{}'.format(self.prefix_BT, tramo['name'])

                geom = tramo['geom']
                points = geom.replace('LINESTRING(', '')
                points = points.replace(')', '')
                points = points.split(',')
                o_position = 1
                o_nsegmento = len(points) - 1
                next_ = None

                for o_position, p in enumerate(points):
                    try:
                        dest = points[o_position + 1]
                        if dest:
                            # Vertex inici
                            o_inicio = '{} 0'.format(p)
                            inicio = o_inicio.split(' ')
                            vertex_inicio = {
                                'x': float(inicio[0]),
                                'y': float(inicio[1])
                            }
                            res_srid_inicio = convert_srid(get_srid(o), (vertex_inicio['x'], vertex_inicio['y']))

                            # Vertex final
                            o_final = '{} 0'.format(dest)
                            final = o_final.split(' ')
                            vertex_final = {
                                'x': float(final[0]),
                                'y': float(final[1])
                            }
                            res_srid_final = convert_srid(get_srid(o), (vertex_final['x'], vertex_final['y']))

                            # SEGMENTO
                            o_segmento = '{}_{}'.format(o_tram, o_position + 1)
                            if len(o_segmento) > 22:
                                o_segmento = o_segmento[-22:]

                            self.output_q.put([
                                o_segmento,                                  # SEGMENTO
                                o_tram,                                      # IDENTIFICADOR DE TRAMO
                                o_position + 1,                              # ORDEN EN LA LISTA DE SEGMENTOS
                                o_nsegmento,                                 # TOTAL SEGMENTOS
                                format_f(res_srid_inicio[0], decimals=3),    # COORDENADA X INICIO
                                format_f(res_srid_inicio[1], decimals=3),    # COORDENADA Y INICIO
                                '0,000',                                     # COORDENADA Z INICIO
                                format_f(res_srid_final[0], decimals=3),     # COORDENADA X FINAL
                                format_f(res_srid_final[1], decimals=3),     # COORDENADA Y FINAL
                                '0,000',                                     # COORDENADA Z FINAL
                            ])
                    except:
                        pass

                self.input_q.task_done()
            except Exception:
                self.input_q.task_done()
                traceback.print_exc()
                if self.raven:
                    self.raven.captureException()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
INVENTARI DE CNMC Centres Transformadors
"""
from __future__ import absolute_import
from datetime import datetime
import traceback

from libcnmc.core import StopMultiprocessBased
from libcnmc.utils import (
    get_id_municipi_from_company, get_forced_elements, adapt_diff, convert_srid, get_srid, format_f,
    convert_spanish_date, format_f_6181, get_codi_actuacio, get_ine
)
from libcnmc.models import F8Res4666
from shapely import wkt

ZONA = {
    'RURAL CONCENTRADA': 'RC',
    'RURAL DISPERSA': 'RD',
    'URBANA': 'U',
    'SEMIURBANA': 'SU'
}


class FB2(StopMultiprocessBased):
    """
    Class that generates the CT file of the 4666
    """
    def __init__(self, **kwargs):
        """
        Class constructor
        :param kwargs: year(generation year), codi_r1 R1 code
        :return: CT
        """
        super(FB2, self).__init__(**kwargs)
        self.year = kwargs.pop('year', datetime.now().year - 1)
        self.codi_r1 = kwargs.pop('codi_r1')
        self.base_object = 'CTS'
        self.report_name = 'Formulario B2: Centros de Transformación'
        self.compare_field = "4666_entregada"
        self.extended = kwargs.get("extended", False)

    def get_sequence(self):
        """
        Method that generates a list of ids to pass to the consummer
        :return: List of ids
        :rtype: list[int]
        """

        search_params = [('id_installacio.name', '!=', 'SE')]
        data_pm = '{0}-01-01'.format(self.year + 1)
        data_baixa = '{0}-01-01'.format(self.year)
        search_params += [('propietari', '=', True),
                          '|', ('data_pm', '=', False),
                               ('data_pm', '<', data_pm),
                          '|',
                          '&', ('data_baixa', '>', data_baixa),
                               ('ct_baixa', '=', True),
                          '|',
                               ('data_baixa', '=', False),
                               ('ct_baixa', '=', False)
                          ]
        # Revisem que si està de baixa ha de tenir la data informada.
        search_params += ['|',
                          '&', ('active', '=', False),
                               ('data_baixa', '!=', False),
                          ('active', '=', True)]

        forced_ids = get_forced_elements(self.connection, "giscedata.cts")

        ids = self.connection.GiscedataCts.search(
            search_params, 0, 0, False, {'active_test': False}
        )

        ids = ids + forced_ids["include"]
        ids = list(set(ids) - set(forced_ids["exclude"]))

        return list(set(ids))

    def get_node_vertex(self, ct_id):
        O = self.connection
        bloc = O.GiscegisBlocsCtat.search([('ct', '=', ct_id)])
        node = ''
        vertex = None
        if bloc:
            bloc = O.GiscegisBlocsCtat.read(bloc[0], ['node', 'vertex'])
            if bloc['node']:
                node = bloc['node'][1]
                if bloc['vertex']:
                    v = O.GiscegisVertex.read(bloc['vertex'][0], ['x', 'y'])
                    vertex = (round(v['x'], 3), round(v['y'], 3))
        return node, vertex

    def get_ine(self, municipi_id):
        O = self.connection
        muni = O.ResMunicipi.read(municipi_id, ['ine', 'dc'])
        return get_ine(O, muni['ine'])

    def get_potencia_trafos(self, id_ct):
        o = self.connection
        res = 0
        ids_trafos = o.GiscedataTransformadorTrafo.search([
            ('ct', '=', id_ct), ('id_estat.cnmc_inventari', '=', True)])
        if ids_trafos:
            for elem in ids_trafos:
                trafo = o.GiscedataTransformadorTrafo.read(
                    elem, ['potencia_nominal'])
                if trafo:
                    res += trafo['potencia_nominal']
        return res

    def consumer(self):
        """
        Method that generates the csv file
        :return: List of arrays
        """
        O = self.connection

        def get_inst_name(element_id):
            vals = self.connection.GiscedataCts.read(
                element_id[0], ['name'])
            return vals['name']

        fields_to_read = [
            'id', 'name', 'cini', 'data_pm', 'tipus_instalacio_cnmc_id', 'tensio_p',
            'id_municipi', 'perc_financament', 'descripcio', 'data_baixa', 'tensio_const',
            self.compare_field, 'node_baixa', 'zona_id', 'node_id', 'potencia',
            'model', 'punt_frontera', 'id_regulatori', 'perc_financament',
        ]

        fields_to_read_obra = [
            'name', 'cini', 'tipo_inversion', 'ccuu', 'codigo_ccaa', 'nivel_tension_explotacion', 'financiado',
            'fecha_aps', 'fecha_baja', 'causa_baja', 'im_ingenieria', 'im_materiales', 'im_obracivil',
            'im_trabajos', 'subvenciones_europeas', 'subvenciones_nacionales', 'subvenciones_prtr', 'avifauna',
            'valor_auditado', 'valor_contabilidad', 'cuenta_contable', 'porcentaje_modificacion',
            'motivacion', 'obra_id', 'identificador_baja',
        ]

        data_pm_limit = '{0}-01-01'.format(self.year + 1)
        data_baixa_limit = '{0}-01-01'.format(self.year)

        while True:
            try:
                item = self.input_q.get()
                if item == 'STOP':
                    self.input_q.task_done()
                    break
                self.progress_q.put(item)

                ct = O.GiscedataCts.read(item, fields_to_read)

                # IDENTIFICADOR_CT
                if ct.get('id_regulatori', False):
                    o_identificador_ct = ct['id_regulatori']
                else:
                    o_identificador_ct = ct['name']

                # Fecha APS
                data_pm = ''
                if ct['data_pm']:
                    data_pm_ct = datetime.strptime(str(ct['data_pm']),
                                                   '%Y-%m-%d')
                    data_pm = data_pm_ct.strftime('%d/%m/%Y')

                # OBRES
                ct_obra = ''
                obra_ti_ct_obj = O.GiscedataProjecteObraTiCts
                obra_ti_ids = obra_ti_ct_obj.search([('element_ti_id', '=', ct['id'])])
                if obra_ti_ids:
                    for obra_ti_id in obra_ti_ids:
                        obra_id_data = obra_ti_ct_obj.read(obra_ti_id, ['obra_id'])
                        obra_id = obra_id_data['obra_id']
                        # Filtre d'obres finalitzades
                        data_finalitzacio_data = O.GiscedataProjecteObra.read(obra_id[0], ['data_finalitzacio'])
                        if data_finalitzacio_data:
                            if data_finalitzacio_data.get('data_finalitzacio', False):
                                data_finalitzacio = data_finalitzacio_data['data_finalitzacio']

                                inici_any = '{}-01-01'.format(self.year)
                                fi_any = '{}-12-31'.format(self.year)
                                if obra_id and data_finalitzacio and inici_any <= data_finalitzacio <= fi_any:
                                    ct_obra = obra_ti_ct_obj.read(obra_ti_id, fields_to_read_obra)
                        if ct_obra:
                            break

                tipo_inversion = ''
                #CAMPS OBRA
                if ct_obra != '':
                    obra_year = data_finalitzacio.split('-')[0]
                    data_pm_year = data_pm.split('/')[2]
                    if ct_obra['tipo_inversion'] != '0' and obra_year != data_pm_year:
                        data_ip = convert_spanish_date(data_finalitzacio)
                    else:
                        data_ip = ''
                    identificador_baja = (
                        get_inst_name(ct_obra['identificador_baja']) if ct_obra['identificador_baja'] else ''
                    )
                    tipo_inversion = ct_obra['tipo_inversion'] or ''
                    im_ingenieria = format_f_6181(ct_obra['im_ingenieria'] or 0.0, float_type='euro')
                    im_materiales = format_f_6181(ct_obra['im_materiales'] or 0.0, float_type='euro')
                    im_obracivil = format_f_6181(ct_obra['im_obracivil'] or 0.0, float_type='euro')
                    im_construccion = str(format_f(
                        float(im_materiales.replace(",", ".")) + float(im_obracivil.replace(",", "."))
                    , 2)).replace(".", ",")
                    im_trabajos = format_f_6181(ct_obra['im_trabajos'] or 0.0, float_type='euro')
                    subvenciones_europeas = format_f_6181(ct_obra['subvenciones_europeas'] or 0.0, float_type='euro')
                    subvenciones_nacionales = format_f_6181(ct_obra['subvenciones_nacionales'] or 0.0, float_type='euro')
                    subvenciones_prtr = format_f_6181(ct_obra['subvenciones_prtr'] or 0.0, float_type='euro')
                    valor_auditado = format_f_6181(ct_obra['valor_auditado'] or 0.0, float_type='euro')
                    motivacion = get_codi_actuacio(O, ct_obra['motivacion'] and ct_obra['motivacion'][0]) if not \
                        ct_obra['fecha_baja'] else ''
                    cuenta_contable = ct_obra['cuenta_contable'] or ''
                    avifauna = int(ct_obra['avifauna'] == True)
                else:
                    data_ip = ''
                    identificador_baja = ''
                    im_ingenieria = ''
                    im_construccion = ''
                    im_trabajos = ''
                    subvenciones_europeas = ''
                    subvenciones_nacionales = ''
                    subvenciones_prtr = ''
                    valor_auditado = ''
                    motivacion = ''
                    cuenta_contable = ''
                    avifauna = ''

                # Si la data APS es igual a l'any de la generació del fitxer,
                # la data IP sortirà en blanc
                if data_ip:
                    data_ip = '' if data_pm and int(data_pm.split('/')[2]) == int(data_ip.split('/')[2]) \
                    else data_ip

                # FINANCIADO
                financiado = ''
                if isinstance(ct.get('perc_financament', False), float):
                    financiado = 100 - ct['perc_financament']

                #CCAA
                #funció per trobar la ccaa desde el municipi
                fun_ccaa = O.ResComunitat_autonoma.get_ccaa_from_municipi
                if ct['id_municipi']:
                    id_municipi = ct['id_municipi'][0]
                else:
                    id_municipi = get_id_municipi_from_company(O)
                comunitat_codi = ''
                if id_municipi:
                    id_comunitat = fun_ccaa(id_municipi)
                    comunitat_vals = O.ResComunitat_autonoma.read(
                        id_comunitat[0], ['codi'])
                    if comunitat_vals:
                        comunitat_codi = comunitat_vals['codi']

                #DATA_BAIXA, CAUSA_BAJA
                if ct['data_baixa']:
                    if ct['data_baixa'] < data_pm_limit:
                        tmp_date = datetime.strptime(
                            ct['data_baixa'], '%Y-%m-%d %H:%M:%S')
                        fecha_baja = tmp_date.strftime('%d/%m/%Y')

                        if int(fecha_baja.split("/")[2]) - int(data_pm.split("/")[2]) >= 40:
                            if identificador_baja != '':
                                causa_baja = 1
                            else:
                                causa_baja = 2
                        else:
                            causa_baja = 3
                    else:
                        fecha_baja = ''
                        causa_baja = 0;
                else:
                    fecha_baja = ''
                    causa_baja = 0;

                #CCUU
                if ct['tipus_instalacio_cnmc_id']:
                    id_ti = ct['tipus_instalacio_cnmc_id'][0]
                    ti = O.GiscedataTipusInstallacio.read(
                        id_ti,
                        ['name'])['name']
                else:
                    ti = ''

                #NODE ALTA
                if ct.get("node_id"):
                    o_node = ct["node_id"][1]
                    node = O.GiscegisNodes.read(ct["node_id"][0], ["geom"])
                    coords = wkt.loads(node["geom"]).coords[0]
                    vertex = [coords[0], coords[1]]
                else:
                    o_node, vertex = self.get_node_vertex(item)
                o_node = o_node.replace('*', '')

                #NODE BAIXA
                if ct["node_baixa"]:
                    o_node_baixa = ct["node_baixa"][1]
                    if o_node_baixa == 0:
                        o_node_baixa = '';
                else:
                    o_node_baixa = '';

                if ct['cini']:
                    cini = ct['cini']
                    if cini[4] == '5' and cini[7] == 'Z' and o_node:
                        o_node_baixa = o_node

                #TENSIO
                try:
                    o_tensio_p = format_f(
                        float(ct['tensio_p']) / 1000.0, decimals=3) or ''
                except:
                    o_tensio_p = ''

                #TENSIO_CONST
                o_tensio_const = ''
                if ct.get('tensio_const', False):
                    o_tensio_const = format_f(float(ct['tensio_const'][1]) / 1000.0, decimals=3) or ''

                if o_tensio_const == o_tensio_p:
                    o_tensio_const = ''

                #POTENCIA
                o_potencia = str(format_f(
                    float(self.get_potencia_trafos(item)), decimals=3)).replace('.',',')

                #X,Y,Z
                res_srid = ['', '']
                if vertex:
                    res_srid = convert_srid(get_srid(O), vertex)

                #MUNICIPI I PROVINCIA
                municipio = ''
                provincia = ''
                if ct.get('id_municipi', False):
                    provincia, municipio = self.get_ine(ct['id_municipi'][0])

                #ZONA
                if 'zona_id' in ct and ct['zona_id']:
                    zona = O.GiscedataCtsZona.read(
                        ct['zona_id'][0], ['name']
                    )
                    tmp_zona = zona.get('name', "")
                    zona_name = ZONA[tmp_zona.upper()]
                else:
                    zona_name = ""

                #PUNT_FRONTERA
                punto_frontera = int(ct['punt_frontera'] == True)

                #MODELO
                modelo = ''
                if ct.get('model', False):
                    modelo = ct['model']

                # Estado
                if ct[self.compare_field]:
                    last_data = ct[self.compare_field]
                    entregada = F8Res4666(**last_data)

                    id_ti = ct['tipus_instalacio_cnmc_id'][0]
                    ti = O.GiscedataTipusInstallacio.read(
                        id_ti,
                        ['name'])['name']

                    actual = F8Res4666(
                        ct['name'],
                        ct['cini'],
                        '',
                        ti,
                        comunitat_codi,
                        '',
                        data_pm,
                        fecha_baja,
                        0
                    )
                    if entregada == actual and fecha_baja == '':
                        estado = '0'
                        if ct_obra:
                            estado = '1'
                    else:
                        self.output_m.put("{} {}".format(ct["name"], adapt_diff(actual.diff(entregada))))
                        estado = '1'
                else:
                    estado = '2'

                # Fecha APS / Estado
                if modelo == 'M':
                    estado = ''
                    data_pm = ''

                if fecha_baja:
                    tipo_inversion = ''

                # L'any 2022 no es declaren subvencions PRTR
                subvenciones_prtr = ''

                if causa_baja == '0':
                    fecha_baja = ''

                if modelo == 'E':
                    tipo_inversion = '0'
                    estado = '1'

                output = [
                    o_identificador_ct,           # IDENTIFICADOR
                    ct['cini'] or '',                   # CINI
                    ct['descripcio'] or '',             # DENOMINACION
                    str(ti),                            # CODIGO_CCUU
                    o_node,                             # NUDO_ALTA
                    o_node_baixa,                       # NUDO_BAJA
                    o_tensio_p,                         # NIVEL TENSION
                    o_tensio_const,                     # TENSION CONSTRUCCION
                    o_potencia,                         # POTENCIA
                    format_f(res_srid[0], decimals=3),  # X
                    format_f(res_srid[1], decimals=3),  # Y
                    '0,000',                            # Z
                    municipio,                          # MUNICIPIO
                    provincia,                          # PROVINCIA
                    comunitat_codi or '',               # CODIGO_CCAA
                    zona_name,                          # ZONA
                    estado,                             # ESTADO
                    modelo,                             # MODELO
                    punto_frontera,                     # PUNTO_FRONTERA
                    data_pm,                            # FECHA APS
                    causa_baja,                         # CAUSA BAJA
                    fecha_baja,                         # FECHA BAJA
                    data_ip,                            # FECHA IP
                    tipo_inversion,                     # TIPO INVERSION
                    im_ingenieria,                      # IM_TRAMITES
                    im_construccion,                    # IM_CONSTRUCCION
                    im_trabajos,                        # IM_TRABAJOS
                    subvenciones_europeas,              # SUBVENCIONES EUROPEAS
                    subvenciones_nacionales,            # SUBVENCIONES NACIONALES
                    subvenciones_prtr,                  # SUBVENCIONES PRTR
                    valor_auditado,                     # VALOR AUDITADO
                    format_f(financiado, decimals=2),   # FINANCIADO
                    cuenta_contable,                    # CUENTA CONTABLE
                    motivacion,                         # MOTIVACION
                    avifauna,                           # AVIFAUNA
                    identificador_baja,                 # ID_BAJA
                ]
                self.output_q.put(output)
                self.input_q.task_done()
            except Exception:
                self.input_q.task_done()
                traceback.print_exc()
                if self.raven:
                    self.raven.captureException()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
INVENTARI DE CNMC Centres Transformadors
"""
from __future__ import absolute_import
from datetime import datetime
import traceback
from libcnmc.core import StopMultiprocessBased
from libcnmc.utils import format_f, get_norm_tension

OPERACIO = {
    'Reserva': '0',
    'Operativo': '1',
}

class FB3_3(StopMultiprocessBased):

    """
    Class that generates the CT file of the 4666
    """
    def __init__(self, **kwargs):
        super(FB3_3, self).__init__(**kwargs)
        self.year = kwargs.pop('year', datetime.now().year - 1)
        self.report_name = 'FB3_3 - TRAFOS-SE'
        self.base_object = 'TRAFOS'

    def get_sequence(self):
        search_params = [
            ('reductor', '=', True),
            ('id_estat.cnmc_inventari', '=', True)
        ]
        return self.connection.GiscedataTransformadorTrafo.search(
            search_params
        )

    def get_estat(self, estat_id):
        o = self.connection
        estat = o.GiscedataTransformadorEstat.read(estat_id, ['codi'])
        if estat['codi'] != 1:
            return 0
        else:
            return 1

    def get_parcs(self, trafo):
        o = self.connection
        res = {'o_costat_alta': '',
               'o_costat_baixa': ''}
        conexions_ids = trafo['conexions']

        con_obj = o.GiscedataTransformadorConnexio
        se_obj = o.GiscedataCtsSubestacions
        parc_obj = o.GiscedataParcs
        tensio_obj = o.GiscedataTensionsTensio

        con_alta, con_baixa = False, False
        for connexio_id in conexions_ids:
            con_data = con_obj.read(connexio_id)
            if con_data['conectada']:
                con_alta = con_data['tensio_primari']
                if con_data['tensio_b1']:
                    con_baixa = con_data['tensio_b1']
                elif con_data['tensio_b2']:
                    con_baixa = con_data['tensio_b2']
                elif con_data['tensio_b3']:
                    con_baixa = con_data['tensio_b3']
                break

        if con_alta and con_baixa:
            tensions_ids = tensio_obj.search([('tipus', '=', 'AT')])
            tensio_alta, tensio_baixa = False, False
            for tensio_id in tensions_ids:
                llindar_inf = tensio_obj.read(tensio_id, ['l_inferior'])['l_inferior']
                llindar_sup = tensio_obj.read(tensio_id, ['l_superior'])['l_superior']
                if llindar_inf <= con_alta <= llindar_sup:
                    tensio_alta = tensio_id
                elif llindar_inf <= con_baixa <= llindar_sup:
                    tensio_baixa = tensio_id

            if tensio_alta and tensio_baixa:
                se_name = trafo['ct'][1]
                se_id = se_obj.search([('name', '=', se_name)])
                parc_id_alta = parc_obj.search([('subestacio_id', '=', se_id), ('tensio_id', '=', tensio_alta)])
                if parc_id_alta:
                    res['o_costat_alta'] = parc_obj.read(parc_id_alta[0], ['name'])['name']
                parc_id_baixa = parc_obj.search([('subestacio_id', '=', se_id), ('tensio_id', '=', tensio_baixa)])
                if parc_id_baixa:
                    res['o_costat_baixa'] = parc_obj.read(parc_id_baixa[0], ['name'])['name']

        return res

    def get_costat_baixa(self, trafo):
        o = self.connection
        res = ''
        if trafo['conexions']:
            con = o.GiscedataTransformadorConnexio.read(trafo['conexions'][0])
            tensio = con['tensio_b1']
            tensio_n = get_norm_tension(o, tensio)
            se_id = trafo['ct'][1]
            parc_id = o.GiscedataParcs.search(
                [
                    ('subestacio_id', '=', se_id),
                    ('tensio_id.tensio', '=', tensio_n)
                ]
            )
            if parc_id:
                res = o.GiscedataParcs.read(parc_id[0], ['name'])['name']
        return res

    def get_operacio(self, id_estat):

        o = self.connection
        operacio = o.GiscedataTransformadorEstat.read(
            id_estat[0], ['operacio']
        )['operacio']
        return operacio

    def consumer(self):
        o = self.connection
        fields_to_read = [
            'ct', 'name', 'cini', 'propietari', 'id_estat',
            'conexions', 'id_operacio', 'data_pm'
        ]
        while True:
            try:
                # generar linies
                item = self.input_q.get()
                if item == 'STOP':
                    self.input_q.task_done()
                    break
                self.progress_q.put(item)
                trafo = o.GiscedataTransformadorTrafo.read(
                    item, fields_to_read
                )
                o_subestacio = trafo['ct'][1]
                o_maquina = trafo['name']
                o_cini = trafo['cini']

                # IDENTIFICADOR_PARQUE_ALTA / IDENTIFICADOR_PARQUE_BAJA
                if trafo.get('conexions', False):
                    o_costat_alta = self.get_parcs(trafo)['o_costat_alta']
                    o_costat_baixa = self.get_parcs(trafo)['o_costat_baixa']

                o_propietat = int(trafo['propietari'])
                o_estat = self.get_estat(trafo['id_estat'][0])

                # AÑO_PS
                o_any_ps = ''
                if trafo.get('data_pm', False):
                    o_any_data = str(trafo['data_pm'])
                    o_any_ps = o_any_data.split('-')[0]


                id_estat = trafo['id_estat']
                if id_estat:
                    operacio = self.get_operacio(id_estat)
                    if operacio:
                        o_operacio = operacio
                    else:
                        o_operacio = 0
                else:
                    o_operacio = 0

                if o_cini:
                    if o_cini[1] == '2' and o_cini[2] == '7' and o_cini[5] == '1' and o_cini[7] == '1':
                        o_operacio = 0

                self.output_q.put([
                    o_subestacio,  # SUBESTACION
                    o_maquina,  # MAQUINA
                    o_cini,  # CINI
                    o_costat_alta,  # PARQUE ALTA
                    o_costat_baixa,  # PARQUE BAJA
                    o_any_ps,  # AÑO INFORMACION
                    o_operacio # OPERACION
                ])
                self.input_q.task_done()
            except Exception:
                self.input_q.task_done()
                traceback.print_exc()
                if self.raven:
                    self.raven.captureException()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
INVENTARIO DE CNMC - Elementos de mejora de fiabilidad
"""
from datetime import datetime
import traceback
from libcnmc.utils import (format_f, convert_srid, get_srid, get_name_ti, format_f_6181, format_ccaa_code, get_ine,
                           adapt_diff)
from libcnmc.core import StopMultiprocessBased
from libcnmc.models import F7Res4666
from shapely import wkt

MODELO = {
    '1': 'I',
    '2': 'M',
    '3': 'D',
    '4': 'E'
}

class FB6(StopMultiprocessBased):

    """
    Class that generates the CT file of the 4666
    """
    def __init__(self, **kwargs):
        """
        Class constructor

        :param year: Year to generate
        :type year: int
        :param codi_r1:
        :type codi_r1:str
        """
        super(FB6, self).__init__(**kwargs)
        self.year = kwargs.pop('year', datetime.now().year - 1)
        self.codi_r1 = kwargs.pop('codi_r1') or ''
        self.report_name = 'FB6 - Elements de millora de fiabilitat'
        self.base_object = 'Elements de millora de fiabilitat'
        self.cod_dis = 'R1-{}'.format(self.codi_r1[-3:])
        self.compare_field = "4666_entregada"
        self.prefix_AT = kwargs.pop('prefix_at', 'A') or 'A'
        self.compare_field = '4666_entregada'

    def get_sequence(self):
        """
                Generates the sequence of ids to pass to the consume function
                :return: List of ids to generate the
                :rtype: list(int)
                """
        data_pm = '%s-01-01' % (self.year + 1)
        data_baixa = '%s-01-01' % self.year

        search_params = [('criteri_regulatori', '!=', 'excloure'),
                         '|', ('data_pm', '=', False),
                              ('data_pm', '<', data_pm),
                         '|', ('data_baixa', '>=', data_baixa),
                              ('data_baixa', '=', False)]

        search_params += [
            ("tipus_element.codi_cnmc", "!=", "T"),
            ('inventari', '=', 'fiabilitat'),
        ]

        return self.connection.GiscedataCellesCella.search(search_params, 0, 0, False, {'active_test': False})

    def get_node_vertex(self, element_name):
        """
        Return the node and the (X,Y) of the Cel·la retrieving this values from
        the Blocs models
        :param element_name: Name of the Cel·la
        :rtype element_name: str
        :return: The Node name and the (X,Y) of the Cel·la
        :rtype: (str or bool, (float, float) or bool)
        """
        o = self.connection
        node = ''
        vertex = False
        bloc_id = []
        model_ok = False
        if element_name:
            # Search on the diferent models
            models = [
                o.GiscegisBlocsInterruptorat,
                o.GiscegisBlocsFusiblesat,
                o.GiscegisBlocsSeccionadorat,
                o.GiscegisBlocsSeccionadorunifilar
            ]
            for model in models:
                bloc_id = model.search([('codi', '=', element_name)])
                if bloc_id:
                    model_ok = model
                    break
            if bloc_id:
                bloc = model_ok.read(
                    bloc_id[0], ['node', 'vertex']
                )
                if bloc.get('node', False):
                    node = bloc['node'][1]
                if bloc.get('vertex', False):
                    v = o.GiscegisVertex.read(bloc['vertex'][0], ['x', 'y'])
                    vertex = (v['x'], v['y'])

        return node, vertex

    def get_ine(self, municipi_id):
        """
        Returns the INE code of the given municipi
        :param municipi_id: Id of the municipi
        :type municipi_id: int
        :return: state, ine municipi
        :rtype:tuple
        """
        O = self.connection
        muni = O.ResMunicipi.read(municipi_id, ['ine'])
        return get_ine(O, muni['ine'])

    def obtenir_camps_linia_at(self, linia_id):
        """
        Gets the data of the line where the cel·la is placed

        :param installacio: Cel·la placement
        :return: Municipi, provincia, tensio of the line
        :rtype: dict
        """

        o = self.connection
        tensio_obj = o.GiscedataTensionsTensio

        res = {
            'id_municipi': '',
            'tensio': '',
            'name': ''
        }

        fields_to_read = ['municipi', 'tensio_id', 'name']
        linia_data = o.GiscedataAtLinia.read(linia_id, fields_to_read)

        if linia_data.get('municipi', False):
            res['id_municipi'] = linia_data['municipi'][0]
        if linia_data.get('tensio_id', False):
            tensio_id = linia_data['tensio_id'][0]
            tensio_data = tensio_obj.read(tensio_id, ['tensio'])
            if tensio_data.get('tensio', False):
                res['tensio'] = format_f(float(tensio_data['tensio']) / 1000.0, decimals=3)
        if linia_data.get('name', False):
            res['name'] = linia_data['name'][0]

        return res

    def get_node_vertex_tram(self, element_name):
        o = self.connection
        tram_name = ""
        if element_name:
            # Search on the diferent models
            models = [
                o.GiscegisBlocsInterruptorat, o.GiscegisBlocsFusiblesat,
                o.GiscegisBlocsSeccionadorat, o.GiscegisBlocsSeccionadorunifilar
            ]
            for model in models:
                bloc_id = model.search([('codi', '=', element_name)])
                if bloc_id:
                    bloc_id = bloc_id[0]
                    bloc = model.read(bloc_id, ['node', 'vertex'])
                    v = o.GiscegisVertex.read(bloc['vertex'][0], ['id'])
                    polver_ids = o.GiscegisPolylineVertex.search(
                        [('vertex', '=', v['id'])]
                    )
                    if polver_ids:
                        poly_id = o.GiscegisPolyline.search(
                            [('vertex_ids', 'in', polver_ids[0])]
                        )[0]
                        edge_id = o.GiscegisEdge.search(
                            [('polyline', '=', poly_id)]
                        )[0]
                        linktemplate = o.GiscegisEdge.read(
                            edge_id, ['id_linktemplate']
                        )['id_linktemplate']
                        tram_id = o.GiscedataAtTram.search(
                            [('name', '=', linktemplate)]
                        )
                        tram_name = o.GiscedataAtTram.read(
                            tram_id, ['name']
                        )[0]['name']
                        return "{}{}".format(self.prefix_AT, tram_name)
        return ""

    def consumer(self):
        """
        Function that generates each line of the file

        :return: None
        """

        O = self.connection

        def get_inst_name(element_id):
            vals = self.connection.GiscedataCellesCella.read(
                element_id, ['name'])
            return vals['name']

        fields_to_read = [
            'installacio', 'cini', 'propietari', 'name', 'tensio', 'node_id', 'perc_financament',
            'tipus_instalacio_cnmc_id', 'punt_frontera', 'tensio_const', 'model',
            'geom', 'tram_id', 'id', 'data_pm', 'data_baixa', self.compare_field,
        ]
        fields_to_read_obra = [
            'name', 'cini', 'tipo_inversion', 'ccuu', 'codigo_ccaa', 'nivel_tension_explotacion', 'elemento_act',
            'financiado', 'fecha_aps', 'fecha_baja', 'causa_baja', 'im_ingenieria', 'im_materiales', 'im_obracivil',
            'im_trabajos', 'subvenciones_europeas', 'subvenciones_nacionales', 'subvenciones_prtr', 'avifauna',
            'valor_auditado', 'valor_contabilidad', 'cuenta_contable', 'porcentaje_modificacion', 'motivacion',
            'obra_id', 'identificador_baja',
        ]

        data_pm_limit = '{0}-01-01'.format(self.year + 1)
        while True:
            try:
                item = self.input_q.get()
                if item == 'STOP':
                    self.input_q.task_done()
                    break
                self.progress_q.put(item)
                cella = O.GiscedataCellesCella.read(
                    item, fields_to_read
                )

                # MODEL
                if cella['model']:
                    modelo = cella['model']
                else:
                    modelo = ''

                # FECHA_APS
                data_pm = ''
                if cella['data_pm']:
                    data_pm_ct = datetime.strptime(str(cella['data_pm']),
                                                   '%Y-%m-%d')
                    data_pm = data_pm_ct.strftime('%d/%m/%Y')

                # OBRES
                cella_obra = ''
                obra_ti_pos_obj = O.GiscedataProjecteObraTiCelles
                obra_ti_ids = obra_ti_pos_obj.search([('element_ti_id', '=', cella['id'])])
                if obra_ti_ids:
                    for obra_ti_id in obra_ti_ids:
                        obra_id_data = obra_ti_pos_obj.read(obra_ti_id, ['obra_id'])
                        obra_id = obra_id_data['obra_id']
                        # Filtre d'obres finalitzades
                        data_finalitzacio_data = O.GiscedataProjecteObra.read(obra_id[0], ['data_finalitzacio'])
                        if data_finalitzacio_data:
                            if data_finalitzacio_data.get('data_finalitzacio', False):
                                data_finalitzacio = data_finalitzacio_data['data_finalitzacio']

                                inici_any = '{}-01-01'.format(self.year)
                                fi_any = '{}-12-31'.format(self.year)
                                if obra_id and data_finalitzacio and inici_any <= data_finalitzacio <= fi_any:
                                    cella_obra = O.GiscedataProjecteObraTiCelles.read(obra_ti_id, fields_to_read_obra)
                        if cella_obra:
                            break

                tipo_inversion = ''
                # CAMPS OBRA
                if cella_obra != '':
                    tipo_inversion = cella_obra['tipo_inversion'] or ''
                    im_ingenieria = format_f_6181(cella_obra['im_ingenieria'] or 0.0, float_type='euro')
                    im_materiales = format_f_6181(cella_obra['im_materiales'] or 0.0, float_type='euro')
                    im_obracivil = format_f_6181(cella_obra['im_obracivil'] or 0.0, float_type='euro')
                    im_trabajos = format_f_6181(cella_obra['im_trabajos'] or 0.0, float_type='euro')
                    identificador_baja = (
                        get_inst_name(cella_obra['identificador_baja'][0]) if cella_obra['identificador_baja'] else ''
                    )
                    im_construccion = str(format_f(
                        float(im_materiales.replace(",", ".")) + float(im_obracivil.replace(",", "."))
                    , 2)).replace(".", ",")
                    subvenciones_europeas = format_f_6181(cella_obra['subvenciones_europeas'] or 0.0, float_type='euro')
                    subvenciones_nacionales = format_f_6181(cella_obra['subvenciones_nacionales'] or 0.0, float_type='euro')
                    subvenciones_prtr = format_f_6181(cella_obra['subvenciones_prtr'] or 0.0, float_type='euro')
                    valor_auditado = format_f_6181(cella_obra['valor_auditado'] or 0.0, float_type='euro')
                    valor_contabilidad = format_f_6181(cella_obra['valor_contabilidad'] or 0.0, float_type='euro')
                    cuenta_contable = cella_obra['cuenta_contable'] or ''
                    avifauna = int(cella_obra['avifauna'] == True)
                    financiado = format_f(cella_obra.get('financiado') or 0.0, 2)
                else:
                    ccuu = ''
                    ccaa = ''
                    im_ingenieria = ''
                    im_materiales = ''
                    im_obracivil = ''
                    im_trabajos = ''
                    im_construccion = ''
                    subvenciones_europeas = ''
                    subvenciones_nacionales = ''
                    valor_auditado = ''
                    valor_contabilidad = ''
                    cuenta_contable = ''
                    identificador_baja = ''
                    financiado = ''
                    avifauna = ''
                    subvenciones_prtr = ''

                o_fiabilitat = cella['name']
                o_cini = cella['cini']
                o_prop = int(cella['propietari'])

                # TRAM, MUNICIPI I PROVINCIA
                o_identificador_elemento = ''
                id_municipi = ''
                linia_data = {}
                ct_data = {}
                if cella.get('installacio', False):
                    installacio_data = cella['installacio']
                    inst_model = installacio_data.split(',')[0]
                    inst_id = int(installacio_data.split(',')[1])
                    model_obj = O.model(inst_model)

                    if inst_model == 'giscedata.at.suport':
                        if cella.get('tram_id', False):
                            tram_data = O.GiscedataAtTram.read(cella['tram_id'][0], ['name', 'id_regulatori'])
                            if tram_data.get('id_regulatori', False):
                                o_identificador_elemento = tram_data['id_regulatori']
                            else:
                                o_identificador_elemento = "{}{}".format(self.prefix_AT, tram_data['name'])
                        suport_data = O.GiscedataAtSuport.read(inst_id, ['linies_at_ids'])
                        if suport_data.get('linies_at_ids', False):
                            linia_id = suport_data['linies_at_ids'][0]
                            linia_data = self.obtenir_camps_linia_at(linia_id)

                    elif inst_model == 'giscedata.cts':
                        ct_data = model_obj.read(inst_id, ['name', 'id_regulatori', 'id_municipi'])
                        if ct_data.get('id_regulatori', False):
                            o_identificador_elemento = ct_data['id_regulatori']
                        else:
                            o_identificador_elemento = ct_data['name']

                else:
                    o_identificador_elemento = self.get_node_vertex_tram(o_fiabilitat)

                o_municipi = ''
                o_provincia = ''
                comunitat_codi = ''
                if linia_data.get('id_municipi', False):
                    id_municipi = linia_data['id_municipi']
                    o_provincia, o_municipi = self.get_ine(id_municipi)
                elif ct_data.get('id_municipi'):
                    id_municipi = ct_data['id_municipi'][0]
                    o_provincia, o_municipi = self.get_ine(id_municipi)

                # funció per trobar la ccaa desde el municipi
                fun_ccaa = O.ResComunitat_autonoma.get_ccaa_from_municipi
                if id_municipi:
                    id_comunitat = fun_ccaa(id_municipi)
                    comunitat_vals = O.ResComunitat_autonoma.read(
                        id_comunitat[0], ['codi'])
                    if comunitat_vals:
                        comunitat_codi = comunitat_vals['codi']

                #FECHA BAJA, CAUSA_BAJA
                if cella['data_baixa']:
                    if cella['data_baixa'] < data_pm_limit:
                        tmp_date = datetime.strptime(
                            cella['data_baixa'], '%Y-%m-%d')
                        fecha_baja = tmp_date.strftime('%d/%m/%Y')
                        if int(fecha_baja.split("/")[2]) - int(data_pm.split("/")[2]) >= 40:
                            if identificador_baja != '':
                                causa_baja = 1
                            else:
                                causa_baja = 2
                        else:
                            causa_baja = 3
                    else:
                        fecha_baja = ''
                        causa_baja = 0
                else:
                    fecha_baja = ''
                    causa_baja = 0

                #CODIGO CCUU
                ti = ''
                if cella.get('tipus_instalacio_cnmc_id', False):
                    id_ti = cella['tipus_instalacio_cnmc_id'][0]
                    ti = O.GiscedataTipusInstallacio.read(id_ti, ['name'])['name']

                #NODE
                if cella.get('node_id'):
                    o_node = cella['node_id'][1]
                    vertex = wkt.loads(cella['geom']).coords[0]
                else:
                    o_node, vertex = self.get_node_vertex(o_fiabilitat)
                o_node = o_node.replace('*', '')

                if cella['tensio']:
                    tensio = O.GiscedataTensionsTensio.read(
                        cella['tensio'][0], ['tensio']
                    )
                    o_tensio = format_f(int(tensio['tensio'])/1000.0, decimals=3)
                else:
                    o_tensio = linia_data.get('tensio', '0,000')

                # TENSIO_CONST
                o_tensio_const = ''
                if cella.get('tensio_const', False):
                    o_tensio_const = format_f(float(cella['tensio_const'][1]) / 1000.0, decimals=3) or ''

                if o_tensio_const == o_tensio:
                    o_tensio_const = ''

                punto_frontera = int(cella['punt_frontera'] == True)
                o_any = self.year
                x = ''
                y = ''
                if vertex:
                    res_srid = convert_srid(get_srid(O), vertex)
                    x = format_f(res_srid[0], decimals=3)
                    y = format_f(res_srid[1], decimals=3)

                # ESTADO
                if cella[self.compare_field] and str(self.year + 1) not in str(data_pm):
                    last_data = cella[self.compare_field]
                    entregada = F7Res4666(**last_data)
                    actual = F7Res4666(
                        cella['name'],
                        cella['cini'],
                        o_identificador_elemento,
                        str(ti),
                        comunitat_codi,
                        data_pm,
                        fecha_baja,
                        0
                    )
                    if entregada == actual and fecha_baja == '':
                        estado = '0'
                        if cella_obra:
                            estado = '1'
                    else:
                        self.output_m.put("{} {}".format(cella["name"], adapt_diff(actual.diff(entregada))))
                        estado = '1'
                else:
                    estado = '2'

                if modelo == 'M':
                    estado = ''
                    data_pm = ''

                if causa_baja in [1, 3]:
                    tipo_inversion = ''

                # L'any 2022 no es declaren subvencions PRTR
                subvenciones_prtr = ''

                if causa_baja == '0':
                    fecha_baja = ''

                if modelo == 'E':
                    tipo_inversion = '0'
                    estado = '1'

                self.output_q.put([
                    o_fiabilitat,   # ELEMENTO FIABILIDAD
                    o_cini,  # CINI
                    o_identificador_elemento,  #IDENTIFICADOR_ELEMENTO
                    o_node,  # NUDO
                    x,              # X
                    y,              # Y
                    '0,000',              # Z
                    o_municipi,     # MUNICIPIO
                    o_provincia,    # PROVINCIA
                    comunitat_codi,     #CCAA
                    str(ti),     #CCUU
                    o_tensio,       # NIVEL TENSION EXPLOTACION
                    o_tensio_const,             # TENSION CONST
                    data_pm,        #FECHA_APS
                    fecha_baja,     #FECHA_BAJA
                    causa_baja,     #CAUSA_BAJA
                    estado,     #ESTADO
                    modelo,      #MODELO
                    punto_frontera,  #PUNT_FRONTERA
                    tipo_inversion,     #TIPO_INVERSION
                    im_ingenieria,    #IM_TRAMITES
                    im_construccion,    #IM_CONSTRUCCION
                    im_trabajos,    #IM_TRABAJOS
                    subvenciones_europeas,      #SUBVENCIONES_EUROPEAS
                    subvenciones_nacionales,     #SUBVENCIONES_NACIONALES
                    subvenciones_prtr,  #SUBVENCIONES_PRTR
                    valor_auditado,    #VALOR_AUDITADO
                    financiado,                 #FINANCIADO
                    cuenta_contable,    #CUENTA
                    avifauna,   #AVIFAUNA
                    identificador_baja,     #IDENTIFICADOR_BAJA
                ])
                self.input_q.task_done()
            except Exception:
                self.input_q.task_done()
                traceback.print_exc()
                if self.raven:
                    self.raven.captureException()



# -*- coding: utf-8 -*-
from datetime import datetime
import traceback

from libcnmc.utils import format_f
from libcnmc.core import StopMultiprocessBased


class F12(StopMultiprocessBased):
    def __init__(self, **kwargs):
        super(F12, self).__init__(**kwargs)
        self.year = kwargs.pop('year', datetime.now().year - 1)
        self.report_name = 'F12 - TRAFOS-CT'
        self.base_object = 'TRAFOS'

    def get_sequence(self):
        data_pm = '%s-01-01' % (self.year + 1)
        data_baixa = '%s-12-31' % self.year
        search_params = [
            ('reductor', '=', False),
            ('id_estat.cnmc_inventari', '=', True)
        ]
        search_params += ['|', ('data_pm', '=', False),
                          ('data_pm', '<', data_pm),
                          '|', ('data_baixa', '>', data_baixa),
                          ('data_baixa', '=', False)
                          ]
        # Revisem que si està de baixa ha de tenir la data informada.
        search_params += ['|',
                          '&', ('active', '=', False),
                               ('data_baixa', '!=', False),
                          ('active', '=', True)]
        return self.connection.GiscedataTransformadorTrafo.search(
            search_params, 0, 0, False, {'active_test': False})

    def get_node(self, trafo_id):
        """
        Returns the Node name of the Trafo
        :param trafo_id: ID of the trafo
        :type trafo_id: int
        :return: Node name of the Trafo
        :rtype: str
        """

        o = self.connection

        trafo = o.GiscedataTransformadorTrafo.read(trafo_id, ["node_id"])
        if trafo.get("node_id"):
            node = trafo.get("node_id")[1]
        else:
            bloc = o.GiscegisBlocsTransformadors.search(
                [('transformadors', '=', trafo_id)])
            node = ''
            if bloc:
                bloc_vals = o.GiscegisBlocsTransformadors.read(
                    bloc[0], ['node'])
                node = bloc_vals['node'][1]

        return node

    def consumer(self):
        o = self.connection
        fields_to_read = [
            'ct', 'name', 'cini', 'potencia_nominal', 'propietari',
            'perdues_buit', 'perdues_curtcircuit_nominal',
        ]
        while True:
            try:
                # generar linies
                item = self.input_q.get()
                if item == "STOP":
                    self.input_q.task_done()
                    break
                self.progress_q.put(item)
                trafo = o.GiscedataTransformadorTrafo.read(
                    item, fields_to_read
                )
                o_ct = trafo['ct'] and trafo['ct'][1] or ''

                o_node = self.get_node(item)
                o_node = o_node.replace('*', '')
                o_cini = trafo['cini'] or ''
                o_maquina = trafo['name']
                o_pot = format_f(
                    float(trafo['potencia_nominal']),
                    decimals=3
                )
                o_perdues_buit = format_f(
                    trafo['perdues_buit'] or 0.0,
                    decimals=3
                )
                o_perdues_nominal = format_f(
                    trafo['perdues_curtcircuit_nominal'] or 0.0,
                    decimals=3
                )
                o_propietari = int(trafo['propietari'])
                o_any = self.year

                self.output_q.put([
                    o_node,             # NUDO
                    o_ct,               # CT
                    o_cini,             # CINI
                    o_maquina,          # MAQUINA
                    o_pot,              # POT MAQUINA
                    o_perdues_buit,     # PERDIDAS VACIO
                    o_perdues_nominal,  # PERDIDAS POT NOMINAL
                    o_propietari,       # PROPIEDAD
                    o_any               # AÑO INFORMACION
                ])
                self.input_q.task_done()
            except Exception:
                self.input_q.task_done()
                traceback.print_exc()
                if self.raven:
                    self.raven.captureException()



from __future__ import absolute_import

from libcnmc.models.cnmcmodel import CNMCModel
from collections import OrderedDict

from libcnmc.models.fields import String, Integer, Decimal


class F5Res4666(CNMCModel):
    """
        Class for fifth file of resolution 4666 (Maquines) (Transformadors)
    """
    schema = OrderedDict([
        ('identificador', String()),
        ('cini', String()),
        ('denominacion', String()),
        ('codigo_ccuu', String()),
        ('codigo_ccaa', Integer()),
        ('tension_primario', Decimal(3)),
        ('tension_secundario', Decimal(3)),
        ('participacion', Decimal(3)),
        ('fecha_aps', String()),
        ('fecha_baja', String()),
        ('capacidad', Decimal(3)),
        ('estado', Integer())
    ])

    @property
    def ref(self):
        return self.store.identificador

    def __cmp__(self, other):
        comp_fields = [
            'cini', 'codigo_ccuu', 'fecha_aps'
        ]
        if self.diff(other, comp_fields):
            return True
        else:
            return False





